@@ -1,5 +1,5 @@
 // ClockTool.cc
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden    (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ClockTool.cc 3758 2004-12-10 01:58:09Z mathias $
+// $Id: ClockTool.cc 4070 2005-06-30 08:38:53Z mathias $
 
 #include "ClockTool.hh"
 
@@ -46,8 +46,7 @@
   #include <time.h>
 #endif
 #include <string>
-#include <iostream>
-using namespace std;
+#include <typeinfo>
 
 class ClockMenuItem: public FbTk::MenuItem {
 public:
@@ -149,10 +148,11 @@ ClockTool::ClockTool(const FbTk::FbWindow &parent,
 
     _FB_USES_NLS;	
 
-    // setup timer to update the graphics each second
+    // setup timer to check the clock every 0.01 second
+    // if nothing has changed, it wont update the graphics
     timeval delay;
-    delay.tv_sec = 1;
-    delay.tv_usec = 0;
+    delay.tv_sec = 0;
+    delay.tv_usec = 100000;
     m_timer.setTimeout(delay);
     FbTk::RefCount<FbTk::Command> update_graphic(new FbTk::SimpleCommand<ClockTool>(*this, 
                                                                                     &ClockTool::updateTime));
@@ -220,6 +220,10 @@ void ClockTool::update(FbTk::Subject *subj) {
         resize(new_width, m_button.height());
         resizeSig().notify();
     }
+
+    if (subj != 0 && typeid(*subj) == typeid(ToolTheme))
+        renderTheme(m_button.alpha());
+
 }
 
 unsigned int ClockTool::borderWidth() const { 
@@ -253,8 +257,6 @@ void ClockTool::updateTime() {
         //        sprintf(time_string, "%d:%d", );
 #endif // HAVE_STRFTIME
     }
-
-    m_button.clear();
 }
 
 // Just change things that affect the size
@@ -265,28 +267,27 @@ void ClockTool::updateSizing() {
 }
 
 void ClockTool::reRender() {
+    if (m_pixmap) 
+        m_screen.imageControl().removeImage(m_pixmap);
+
     if (m_theme.texture().usePixmap()) {
-        if (m_pixmap) 
-            m_screen.imageControl().removeImage(m_pixmap);
         m_pixmap = m_screen.imageControl().renderImage(width(), height(),
                                                        m_theme.texture());
         m_button.setBackgroundPixmap(m_pixmap);
-    }
-}
-
-void ClockTool::renderTheme(unsigned char alpha) {
-    if (!m_theme.texture().usePixmap()) {
-        if (m_pixmap)
-            m_screen.imageControl().removeImage(m_pixmap);
+    } else {
         m_pixmap = 0;
         m_button.setBackgroundColor(m_theme.texture().color());
-    } else {
-        reRender();
     }
+}
 
+
+void ClockTool::renderTheme(unsigned char alpha) {
+    m_button.setAlpha(alpha);
     m_button.setJustify(m_theme.justify());
+
+    reRender();
+
     m_button.setBorderWidth(m_theme.border().width());
     m_button.setBorderColor(m_theme.border().color());
-    m_button.setAlpha(alpha);
     m_button.clear();
 }
