@@ -1,5 +1,5 @@
 // Ewmh.cc for fluxbox
-// Copyright (c) 2002-2004 Henrik Kinnunen (fluxgen at user.sourceforge.net)
+// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Ewmh.cc 3815 2005-01-05 05:37:06Z simonb $
+// $Id: Ewmh.cc 4074 2005-07-04 18:19:53Z fluxgen $
 
 #include "Ewmh.hh"
 
@@ -92,7 +92,7 @@ void Ewmh::initForScreen(BScreen &screen) {
         XChangeProperty(disp, wincheck, m_net_supporting_wm_check, XA_WINDOW, 32,
                         PropModeReplace, (unsigned char *) &wincheck, 1);
 
-        XChangeProperty(disp, wincheck, m_net_wm_name, XA_STRING, 8,
+        XChangeProperty(disp, wincheck, m_net_wm_name, utf8_string, 8,
                         PropModeReplace, (unsigned char *) "Fluxbox", strlen("Fluxbox"));
     }
 
@@ -241,7 +241,8 @@ void Ewmh::setupFrame(FluxboxWindow &win) {
             win.setWorkspace(desktop);
 
         XFree(data);
-    }
+    } else 
+        updateWorkspace(win);
 
 }
 
@@ -265,6 +266,10 @@ void Ewmh::updateFocusedWindow(BScreen &screen, Window win) {
                                        (unsigned char *)&win, 1);
 }
 
+void Ewmh::updateClientClose(WinClient &winclient){
+    updateClientList(winclient.screen());
+}
+
 void Ewmh::updateClientList(BScreen &screen) {
     size_t num=0;
 
@@ -283,8 +288,8 @@ void Ewmh::updateClientList(BScreen &screen) {
 
     }
     // and count icons
-    BScreen::Icons::const_iterator icon_it = screen.getIconList().begin();
-    BScreen::Icons::const_iterator icon_it_end = screen.getIconList().end();
+    BScreen::Icons::const_iterator icon_it = screen.iconList().begin();
+    BScreen::Icons::const_iterator icon_it_end = screen.iconList().end();
     for (; icon_it != icon_it_end; ++icon_it) {
         num += (*icon_it)->numClients();
     }
@@ -322,7 +327,7 @@ void Ewmh::updateClientList(BScreen &screen) {
     }
 
     // plus iconified windows
-    icon_it = screen.getIconList().begin();
+    icon_it = screen.iconList().begin();
     for (; icon_it != icon_it_end; ++icon_it) {
         FluxboxWindow::ClientList::iterator client_it = (*icon_it)->clientList().begin();
         FluxboxWindow::ClientList::iterator client_it_end = (*icon_it)->clientList().end();
@@ -415,7 +420,7 @@ void Ewmh::updateCurrentWorkspace(BScreen &screen) {
      * This MUST be set and updated by the Window Manager.
      *
      */
-    unsigned int workspace = screen.currentWorkspaceID();
+    unsigned long workspace = screen.currentWorkspaceID();
     screen.rootWindow().changeProperty(m_net_current_desktop,
                                        XA_CARDINAL, 32,
                                        PropModeReplace,
@@ -432,7 +437,7 @@ void Ewmh::updateWorkspaceCount(BScreen &screen) {
      * Window Manager to indicate the number of virtual
      * desktops.
      */
-    unsigned int numworkspaces = screen.getCount();
+    unsigned long numworkspaces = screen.numberOfWorkspaces();
     screen.rootWindow().changeProperty(m_net_number_of_desktops,
                                        XA_CARDINAL, 32,
                                        PropModeReplace,
@@ -450,7 +455,7 @@ void Ewmh::updateViewPort(BScreen &screen) {
      * desktops, this MUST always be set to (0,0).
      *
      */
-    int value[2] = {0, 0}; // we dont support large desktops
+    long value[2] = {0, 0}; // we dont support large desktops
     screen.rootWindow().changeProperty(m_net_desktop_viewport,
                                        XA_CARDINAL, 32,
                                        PropModeReplace,
@@ -470,7 +475,7 @@ void Ewmh::updateGeometry(BScreen &screen) {
      * Window Manager.
      *
      */
-    int value[2] = {screen.width(), screen.height()};
+    long value[2] = {screen.width(), screen.height()};
     screen.rootWindow().changeProperty(m_net_desktop_geometry,
                                        XA_CARDINAL, 32,
                                        PropModeReplace,
@@ -497,8 +502,8 @@ void Ewmh::updateWorkarea(BScreen &screen) {
      * Not sure how to handle xinerama stuff here.
      * So i'm just doing this on the first head.
      */
-    unsigned int *coords = new unsigned int[4*screen.getCount()];
-    for (unsigned int i=0; i<screen.getCount()*4; i+=4) {
+    unsigned long *coords = new unsigned long[4*screen.numberOfWorkspaces()];
+    for (unsigned int i=0; i < screen.numberOfWorkspaces()*4; i+=4) {
         // x, y
         coords[i] = screen.maxLeft(0);
         coords[i + 1] = screen.maxTop(0);
@@ -511,7 +516,7 @@ void Ewmh::updateWorkarea(BScreen &screen) {
                                        XA_CARDINAL, 32,
                                        PropModeReplace,
                                        (unsigned char *)coords,
-                                       4*screen.getCount());
+                                       4 * screen.numberOfWorkspaces());
 
     delete[] coords;
 }
@@ -521,7 +526,9 @@ void Ewmh::updateState(FluxboxWindow &win) {
     // TODO: should we update the _NET_WM_ALLOWED_ACTIONS
     //       here too?
     
-    std::vector<unsigned int> state;
+    typedef std::vector<unsigned int> StateVec;
+
+    StateVec state;
 
     if (win.isStuck())
         state.push_back(m_net_wm_state_sticky);
@@ -531,10 +538,10 @@ void Ewmh::updateState(FluxboxWindow &win) {
         state.push_back(m_net_wm_state_below);
     if (win.layerNum() == Fluxbox::instance()->getAboveDockLayer())
         state.push_back(m_net_wm_state_above);
-    if (win.isFocusHidden()) {
+    if (win.isIconic())
         state.push_back(m_net_wm_state_hidden);
+    if (win.isIconHidden())
         state.push_back(m_net_wm_state_skip_taskbar);
-    }
     if (win.isFullscreen()) {
         state.push_back(m_net_wm_state_fullscreen);
     }
@@ -544,9 +551,31 @@ void Ewmh::updateState(FluxboxWindow &win) {
     
     it = win.clientList().begin();
     for (; it != it_end; ++it) {
-        if (!state.empty())
+        
+        // search the old states for _NET_WM_STATE_SKIP_PAGER and append it
+        // to the current state, so it wont get deleted by us.
+        StateVec client_state(state);
+        Atom ret_type;
+        int fmt;
+        unsigned long nitems, bytes_after;
+        unsigned char *data = 0;
+
+        (*it)->property(m_net_wm_state, 0, 0x7fffffff, False, XA_ATOM,
+                                 &ret_type, &fmt, &nitems, &bytes_after,
+                                 &data);
+        if (data) {
+            Atom *old_states = (Atom *)data;
+            for (unsigned long i=0; i < nitems; ++i)
+                if (old_states[i] == m_net_wm_state_skip_pager) {
+                    client_state.push_back(m_net_wm_state_skip_pager);
+                }
+            XFree(data);
+        }
+
+        if (!client_state.empty())
             (*it)->changeProperty(m_net_wm_state, XA_ATOM, 32, PropModeReplace,
-                                  reinterpret_cast<unsigned char*>(&state.front()), state.size());
+                                  reinterpret_cast<unsigned char*>(&client_state.front()), 
+                                  client_state.size());
         else
             (*it)->deleteProperty(m_net_wm_state);
     }
@@ -568,7 +597,8 @@ void Ewmh::updateHints(FluxboxWindow &win) {
 }
 
 void Ewmh::updateWorkspace(FluxboxWindow &win) {
-    int workspace = win.workspaceNumber();
+    long workspace = win.isInitialized() ? win.workspaceNumber() : win.screen().currentWorkspaceID();
+
     if (win.isStuck())
         workspace = 0xFFFFFFFF; // appear on all desktops/workspaces
 
@@ -608,7 +638,7 @@ bool Ewmh::checkClientMessage(const XClientMessageEvent &ce,
         // which doesn't apply here (so borrow the variable :) )
         screen = &fbwin->screen();
         // valid workspace number?
-        if (static_cast<unsigned int>(ce.data.l[0]) < screen->getCount())
+        if (static_cast<unsigned int>(ce.data.l[0]) < screen->numberOfWorkspaces())
             screen->sendToWorkspace(ce.data.l[0], fbwin, false);
 
         return true;
@@ -638,20 +668,20 @@ bool Ewmh::checkClientMessage(const XClientMessageEvent &ce,
 
         // no need to alter number of desktops if they are the same
         // or if requested number of workspace is less than zero
-        if (screen->getCount() == static_cast<unsigned int>(ce.data.l[0]) ||
+        if (screen->numberOfWorkspaces() == static_cast<unsigned int>(ce.data.l[0]) ||
             ce.data.l[0] < 0)
             return true;
 
-        if (screen->getCount() > static_cast<unsigned int>(ce.data.l[0])) {
+        if (screen->numberOfWorkspaces() > static_cast<unsigned int>(ce.data.l[0])) {
             // remove last workspace until we have
             // the same number of workspaces
-            while (screen->getCount() != static_cast<unsigned int>(ce.data.l[0])) {
+            while (screen->numberOfWorkspaces() != static_cast<unsigned int>(ce.data.l[0])) {
                 screen->removeLastWorkspace();
-                if (screen->getCount() == 1) // must have at least one workspace
+                if (screen->numberOfWorkspaces() == 1) // must have at least one workspace
                     break;
             }
         } else { // add workspaces to screen until workspace count match the requested size
-            while (screen->getCount() != static_cast<unsigned int>(ce.data.l[0])) {
+            while (screen->numberOfWorkspaces() != static_cast<unsigned int>(ce.data.l[0])) {
                 screen->addWorkspace();
             }
         }
@@ -663,7 +693,7 @@ bool Ewmh::checkClientMessage(const XClientMessageEvent &ce,
         // ce.data.l[0] = workspace number
 
         // prevent out of range value
-        if (static_cast<unsigned int>(ce.data.l[0]) >= screen->getCount())
+        if (static_cast<unsigned int>(ce.data.l[0]) >= screen->numberOfWorkspaces())
             return true;
         screen->changeWorkspaceID(ce.data.l[0]);
         return true;
@@ -706,9 +736,10 @@ bool Ewmh::checkClientMessage(const XClientMessageEvent &ce,
         // ce.data.l[2] = y
         // ce.data.l[3] = width
         // ce.data.l[4] = height
-        // TODO: gravity and flags
-        winclient->fbwindow()->moveResize(ce.data.l[1], ce.data.l[2],
-                                          ce.data.l[3], ce.data.l[4]);
+        // TODO: flags
+        int win_gravity=ce.data.l[0] & 0xFF;
+        winclient->fbwindow()->moveResizeForClient(ce.data.l[1], ce.data.l[2],
+                                          ce.data.l[3], ce.data.l[4], win_gravity);
         return true;
     }
 
@@ -770,6 +801,7 @@ void Ewmh::createAtoms() {
     m_net_wm_state_fullscreen = XInternAtom(disp, "_NET_WM_STATE_FULLSCREEN", False);
     m_net_wm_state_hidden = XInternAtom(disp, "_NET_WM_STATE_HIDDEN", False);
     m_net_wm_state_skip_taskbar = XInternAtom(disp, "_NET_WM_STATE_SKIP_TASKBAR", False);
+    m_net_wm_state_skip_pager = XInternAtom(disp, "_NET_WM_STATE_SKIP_PAGER", False);
     m_net_wm_state_above = XInternAtom(disp, "_NET_WM_STATE_ABOVE", False);
     m_net_wm_state_below = XInternAtom(disp, "_NET_WM_STATE_BELOW", False);
 
@@ -793,6 +825,7 @@ void Ewmh::createAtoms() {
     m_net_wm_handled_icons = XInternAtom(disp, "_NET_WM_HANDLED_ICONS", False);
 
     m_net_wm_ping = XInternAtom(disp, "_NET_WM_PING", False);
+    utf8_string = XInternAtom(disp, "UTF8_STRING", False);
 }
 
 
@@ -852,9 +885,7 @@ void Ewmh::setState(FluxboxWindow &win, Atom state, bool value) {
         if ((value && !win.isFullscreen()) ||
             (!value && win.isFullscreen()))
         setFullscreen(win, value);
-    } else if (state == m_net_wm_state_hidden ||
-               state == m_net_wm_state_skip_taskbar) {
-        win.setFocusHidden(value);
+    } else if (state == m_net_wm_state_skip_taskbar) {
         win.setIconHidden(value);
     } else if (state == m_net_wm_state_below) {  // bottom layer
         if (value)
@@ -867,7 +898,6 @@ void Ewmh::setState(FluxboxWindow &win, Atom state, bool value) {
             win.moveToLayer(Fluxbox::instance()->getAboveDockLayer());
         else
             win.moveToLayer(Fluxbox::instance()->getNormalLayer());
-
     }
 }
 
@@ -883,9 +913,7 @@ void Ewmh::toggleState(FluxboxWindow &win, Atom state) {
         win.maximizeVertical();
     } else if (state == m_net_wm_state_fullscreen) { // fullscreen
         setFullscreen(win, getState(win) == 0); // toggle current state
-    } else if (state == m_net_wm_state_hidden ||
-               state == m_net_wm_state_skip_taskbar) {
-        win.setFocusHidden(!win.isFocusHidden());
+    } else if (state == m_net_wm_state_skip_taskbar) {
         win.setIconHidden(!win.isIconHidden());
     } else if (state == m_net_wm_state_below) { // bottom layer
         if (win.layerNum() == Fluxbox::instance()->getBottomLayer())
