@@ -1,5 +1,5 @@
 // TextureRender.cc for fluxbox
-// Copyright (c) 2002 - 2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // from Image.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: TextureRender.cc 3700 2004-11-19 11:37:27Z mathias $
+// $Id: TextureRender.cc 4097 2005-09-03 14:44:07Z mathias $
 
 #include "TextureRender.hh"
 
@@ -62,16 +62,19 @@ TextureRender::TextureRender(ImageControl &imgctrl,
     width(static_cast<signed>((w > 0 ? w : 1))), height(static_cast<signed>(h > 0 ? h : 1)),
     xtable(0), ytable(0) {
 
+    unsigned int texture_max_width = WidthOfScreen(ScreenOfDisplay(FbTk::App::instance()->display(), imgctrl.screenNumber())) * 2;
+    unsigned int texture_max_height = HeightOfScreen(ScreenOfDisplay(FbTk::App::instance()->display(), imgctrl.screenNumber())) * 2;
+    
     _FB_USES_NLS;
     // clamp to "normal" size
-    if (width > 3200) {
+    if (width > texture_max_width) {
         cerr<<"TextureRender: "<<_FBTKTEXT(Error, BigWidth, "Warning! Width > 3200 setting Width = 3200", "Image width seems too big, clamping")<<endl;
-        width = 3200;
+        width = texture_max_width;
     }
 
-    if (height > 3200) {
+    if (height > texture_max_height) {
         cerr<<"TextureRender: "<<_FBTKTEXT(Error, BigHeight, "Warning! Height > 3200 setting Height = 3200", "Image height seems too big, clamping")<<endl;
-        height = 3200;
+        height = texture_max_height;
     }
 
 
@@ -113,7 +116,7 @@ void TextureRender::allocateColorTables() {
 
     if (red == 0) {
         char sbuf[128];
-        sprintf(sbuf, "%d", size);
+        sprintf(sbuf, "%ld", (long int) size);
         throw std::string("TextureRender::TextureRender(): " + 
               std::string(_FBTKTEXT(Error, OutOfMemoryRed, "Out of memory while allocating red buffer.", "")) + string(sbuf));
     }
@@ -122,7 +125,7 @@ void TextureRender::allocateColorTables() {
     green = FB_new_nothrow unsigned char[size];
     if (green == 0) {
         char sbuf[128];
-        sprintf(sbuf, "%d", size);
+        sprintf(sbuf, "%ld", (long int) size);
         throw std::string("TextureRender::TextureRender(): " + 
               std::string(_FBTKTEXT(Error, OutOfMemoryGreen, "Out of memory while allocating green buffer.", ""))+ string(sbuf));
     }
@@ -130,7 +133,7 @@ void TextureRender::allocateColorTables() {
     blue = FB_new_nothrow unsigned char[size];
     if (blue == 0) {
         char sbuf[128];
-        sprintf(sbuf, "%d", size);
+        sprintf(sbuf, "%ld", (long int) size);
         throw std::string("TextureRender::TextureRender(): " +
               std::string(_FBTKTEXT(Error, OutOfMemoryBlue, "Out of memory while allocating blue buffer.", ""))+ string(sbuf));
     }
