@@ -1,5 +1,5 @@
 // MenuCreator.cc for Fluxbox
-// Copyright (c) 2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2004 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden    (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: MenuCreator.cc 3732 2004-11-30 01:26:00Z mathias $
+// $Id: MenuCreator.cc 4110 2005-09-14 20:28:15Z fluxgen $
 
 #include "MenuCreator.hh"
 
@@ -29,6 +29,7 @@
 #include "fluxbox.hh"
 #include "CommandParser.hh"
 #include "Window.hh"
+#include "WindowCmd.hh"
 
 #include "FbMenu.hh"
 #include "IconMenu.hh"
@@ -46,24 +47,19 @@
 #include "FbTk/MacroCommand.hh"
 #include "FbTk/SimpleCommand.hh"
 #include "FbTk/StringUtil.hh"
-#include "FbTk/Directory.hh"
+#include "FbTk/FileUtil.hh"
 #include "FbTk/MenuSeparator.hh"
 #include "FbTk/MenuIcon.hh"
 
 #include <iostream>
 using namespace std;
 
-template <>
-void LayerMenuItem<FluxboxWindow>::click(int button, int time) {
-    m_object->moveToLayer(m_layernum);
-}
-
-static void createStyleMenu(FbTk::Menu &parent, const std::string &label, 
-                                   const std::string &directory) {
+static void createStyleMenu(FbTk::Menu &parent, const std::string &label,
+                            const std::string &directory) {
     // perform shell style ~ home directory expansion
     string stylesdir(FbTk::StringUtil::expandFilename(directory));
 
-    if (!FbTk::Directory::isDirectory(stylesdir))
+    if (!FbTk::FileUtil::isDirectory(stylesdir.c_str()))
         return;
 
     FbTk::Directory dir(stylesdir.c_str());
@@ -81,25 +77,25 @@ static void createStyleMenu(FbTk::Menu &parent, const std::string &label,
         std::string style(stylesdir + '/' + filelist[file_index]);
         // add to menu only if the file is a regular file, and not a
         // .file or a backup~ file
-        if ((FbTk::Directory::isRegularFile(style) &&
+        if ((FbTk::FileUtil::isRegularFile(style.c_str()) &&
              (filelist[file_index][0] != '.') &&
              (style[style.length() - 1] != '~')) ||
-            FbTk::Directory::isRegularFile(style + "/theme.cfg") ||
-            FbTk::Directory::isRegularFile(style + "/style.cfg"))
+            FbTk::FileUtil::isRegularFile((style + "/theme.cfg").c_str()) ||
+            FbTk::FileUtil::isRegularFile((style + "/style.cfg").c_str()))
             parent.insert(new StyleMenuItem(filelist[file_index], style));
-    } 
+    }
     // update menu graphics
-    parent.update();
+    parent.updateMenu();
     Fluxbox::instance()->saveMenuFilename(stylesdir.c_str());
 
 }
 
-static void createRootCmdMenu(FbTk::Menu &parent, const string &label, 
+static void createRootCmdMenu(FbTk::Menu &parent, const string &label,
                                 const string &directory, const string &cmd) {
     // perform shell style ~ home directory expansion
     string rootcmddir(FbTk::StringUtil::expandFilename(directory));
 
-    if (!FbTk::Directory::isDirectory(rootcmddir))
+    if (!FbTk::FileUtil::isDirectory(rootcmddir.c_str()))
         return;
 
     FbTk::Directory dir(rootcmddir.c_str());
@@ -114,17 +110,17 @@ static void createRootCmdMenu(FbTk::Menu &parent, const string &label,
 
     // for each file in directory add filename and path to menu
     for (size_t file_index = 0; file_index < dir.entries(); file_index++) {
-        
+
         string rootcmd(rootcmddir+ '/' + filelist[file_index]);
         // add to menu only if the file is a regular file, and not a
         // .file or a backup~ file
-        if ((FbTk::Directory::isRegularFile(rootcmd) &&
+        if ((FbTk::FileUtil::isRegularFile(rootcmd.c_str()) &&
              (filelist[file_index][0] != '.') &&
-             (rootcmd[rootcmd.length() - 1] != '~'))) 
+             (rootcmd[rootcmd.length() - 1] != '~')))
             parent.insert(new RootCmdMenuItem(filelist[file_index], rootcmd, cmd));
     }
     // update menu graphics
-    parent.update();
+    parent.updateMenu();
     Fluxbox::instance()->saveMenuFilename(rootcmddir.c_str());
 
 }
@@ -147,6 +143,19 @@ private:
     FbTk::Menu *m_menu;
 };
 
+class MenuContext: public LayerObject {
+public:
+    void moveToLayer(int layer_number) {
+        if (WindowCmd<void>::window() == 0)
+            return;
+        WindowCmd<void>::window()->moveToLayer(layer_number);
+    }
+    int layerNumber() const {
+        if (WindowCmd<void>::window() == 0)
+            return -1;
+        return WindowCmd<void>::window()->layerItem().getLayerNum();
+    }
+};
 
 static void translateMenuItem(Parser &parse, ParseItem &item);
 
@@ -172,10 +181,10 @@ static void translateMenuItem(Parser &parse, ParseItem &pitem) {
 
     const int screen_number = menu.screenNumber();
     _FB_USES_NLS;
-    
+
     if (str_key == "end") {
         return;
-    } else if (str_key == "nop") { 
+    } else if (str_key == "nop") {
         menu.insert(str_label.c_str());
     } else if (str_key == "icons") {
         FbTk::Menu *submenu = MenuCreator::createMenuType("iconmenu", menu.screenNumber());
@@ -195,7 +204,7 @@ static void translateMenuItem(Parser &parse, ParseItem &pitem) {
         // execute and hide menu
         using namespace FbTk;
         RefCount<Command> exec_cmd(CommandParser::instance().parseLine("exec " + str_cmd));
-        RefCount<Command> hide_menu(new SimpleCommand<FbTk::Menu>(menu, 
+        RefCount<Command> hide_menu(new SimpleCommand<FbTk::Menu>(menu,
                                                                   &Menu::hide));
         MacroCommand *exec_and_hide = new FbTk::MacroCommand();
         exec_and_hide->add(hide_menu);
@@ -205,7 +214,7 @@ static void translateMenuItem(Parser &parse, ParseItem &pitem) {
     } else if (str_key == "macrocmd") {
         using namespace FbTk;
         RefCount<Command> macro_cmd(CommandParser::instance().parseLine("macrocmd " + str_cmd));
-        RefCount<Command> hide_menu(new SimpleCommand<FbTk::Menu>(menu, 
+        RefCount<Command> hide_menu(new SimpleCommand<FbTk::Menu>(menu,
                                                                   &Menu::hide));
         MacroCommand *exec_and_hide = new FbTk::MacroCommand();
         exec_and_hide->add(hide_menu);
@@ -218,7 +227,7 @@ static void translateMenuItem(Parser &parse, ParseItem &pitem) {
         BScreen *screen = Fluxbox::instance()->findScreen(screen_number);
         if (screen != 0)
             menu.insert(str_label.c_str(), &screen->configMenu());
-    } // end of config                
+    } // end of config
     else if (str_key == "include") { // include
 
         // this will make sure we dont get stuck in a loop
@@ -229,7 +238,7 @@ static void translateMenuItem(Parser &parse, ParseItem &pitem) {
         safe_counter++;
 
         string newfile = FbTk::StringUtil::expandFilename(str_label);
-        if (FbTk::Directory::isDirectory(newfile)) {
+        if (FbTk::FileUtil::isDirectory(newfile.c_str())) {
             // inject every file in this directory into the current menu
             FbTk::Directory dir(newfile.c_str());
 
@@ -241,7 +250,7 @@ static void translateMenuItem(Parser &parse, ParseItem &pitem) {
             for (size_t file_index = 0; file_index < dir.entries(); file_index++) {
                 std::string thisfile(newfile + '/' + filelist[file_index]);
 
-                if (FbTk::Directory::isRegularFile(thisfile) &&
+                if (FbTk::FileUtil::isRegularFile(thisfile.c_str()) &&
                         (filelist[file_index][0] != '.') &&
                         (thisfile[thisfile.length() - 1] != '~')) {
                     MenuCreator::createFromFile(thisfile, menu, false);
@@ -259,7 +268,7 @@ static void translateMenuItem(Parser &parse, ParseItem &pitem) {
 
     } // end of include
     else if (str_key == "submenu") {
-            
+
         FbTk::Menu *submenu = MenuCreator::createMenu("", screen_number);
         if (submenu == 0)
             return;
@@ -270,32 +279,32 @@ static void translateMenuItem(Parser &parse, ParseItem &pitem) {
             submenu->setLabel(str_label.c_str());
 
         parseMenu(parse, *submenu);
-        submenu->update();
+        submenu->updateMenu();
         menu.insert(str_label.c_str(), submenu);
         // save to screen list so we can delete it later
         BScreen *screen = Fluxbox::instance()->findScreen(screen_number);
-        if (screen != 0)                    
+        if (screen != 0)
             screen->saveMenu(*submenu);
 
     } // end of submenu
     else if (str_key == "stylesdir" || str_key == "stylesmenu") {
-        createStyleMenu(menu, str_label, 
+        createStyleMenu(menu, str_label,
                         str_key == "stylesmenu" ? str_cmd : str_label);
     } // end of stylesdir
     else if (str_key == "themesdir" || str_key == "themesmenu") {
-        createStyleMenu(menu, str_label, 
+        createStyleMenu(menu, str_label,
                         str_key == "themesmenu" ? str_cmd : str_label);
     } // end of themesdir
-    else if (str_key == "wallpapers" || str_key == "wallpapermenu" || 
+    else if (str_key == "wallpapers" || str_key == "wallpapermenu" ||
              str_key == "rootcommands") {
-         createRootCmdMenu(menu, str_label, str_label, 
+         createRootCmdMenu(menu, str_label, str_label,
                           str_cmd == "" ? "fbsetbg" : str_cmd);
     } // end of wallpapers
     else if (str_key == "workspaces") {
         BScreen *screen = Fluxbox::instance()->findScreen(screen_number);
         if (screen != 0) {
-            screen->getWorkspacemenu().setInternalMenu();
-            menu.insert(str_label.c_str(), &screen->getWorkspacemenu());
+            screen->workspaceMenu().setInternalMenu();
+            menu.insert(str_label.c_str(), &screen->workspaceMenu());
         }
     } else if (str_key == "separator") {
         menu.insert(new FbTk::MenuSeparator());
@@ -326,20 +335,20 @@ static void translateMenuItem(Parser &parse, ParseItem &pitem) {
 }
 
 
-static void parseWindowMenu(Parser &parse, FbTk::Menu &menu, FluxboxWindow &win) {
+static void parseWindowMenu(Parser &parse, FbTk::Menu &menu) {
 
     ParseItem pitem(&menu);
     while (!parse.eof()) {
         pitem.load(parse);
-        if (MenuCreator::createWindowMenuItem(pitem.key(), pitem.label(), menu, win))
+        if (MenuCreator::createWindowMenuItem(pitem.key(), pitem.label(), menu))
             continue;
 
         if (pitem.key() == "end") {
             return;
         } else if (pitem.key() == "submenu") {
             FbTk::Menu *submenu = MenuCreator::createMenu(pitem.label(), menu.screenNumber());
-            parseWindowMenu(parse, *submenu, win);
-            submenu->update();
+            parseWindowMenu(parse, *submenu);
+            submenu->updateMenu();
             menu.insert(pitem.label().c_str(), submenu);
 
         } else { // try non window menu specific stuff
@@ -353,8 +362,8 @@ FbTk::Menu *MenuCreator::createMenu(const std::string &label, int screen_number)
     if (screen == 0)
         return 0;
 
-    FbTk::Menu *menu = new FbMenu(screen->menuTheme(), 
-                                  screen->imageControl(), 
+    FbTk::Menu *menu = new FbMenu(screen->menuTheme(),
+                                  screen->imageControl(),
                                   *screen->layerManager().
                                   getLayer(Fluxbox::instance()->getMenuLayer()));
     if (!label.empty())
@@ -390,7 +399,7 @@ FbTk::Menu *MenuCreator::createFromFile(const std::string &filename, int screen_
     std::string label;
     if (require_begin && !getStart(parser, label))
         return 0;
-    
+
     FbTk::Menu *menu = createMenu(label, screen_number);
     if (menu != 0)
         parseMenu(parser, *menu);
@@ -416,9 +425,9 @@ bool MenuCreator::createFromFile(const std::string &filename,
 }
 
 
-bool MenuCreator::createFromFile(const std::string &filename, 
-                                 FbTk::Menu &inject_into, 
-                                 FluxboxWindow &win, bool require_begin) {
+bool MenuCreator::createWindowMenuFromFile(const std::string &filename,
+                                           FbTk::Menu &inject_into,
+                                           bool require_begin) {
     std::string real_filename = FbTk::StringUtil::expandFilename(filename);
     FbMenuParser parser(real_filename);
     if (!parser.isLoaded())
@@ -429,7 +438,7 @@ bool MenuCreator::createFromFile(const std::string &filename,
     if (require_begin && !getStart(parser, label))
         return false;
 
-    parseWindowMenu(parser, inject_into, win);
+    parseWindowMenu(parser, inject_into);
     return true;
 }
 
@@ -442,26 +451,56 @@ FbTk::Menu *MenuCreator::createMenuType(const std::string &type, int screen_num)
         return new IconMenu(*screen);
     } else if (type == "workspacemenu") {
         return new WorkspaceMenu(*screen);
+    } else if (type == "windowmenu") {
+        FbTk::Menu *menu = screen->createMenu("");
+
+        menu->removeAll(); // clear old items
+        menu->disableTitle(); // not titlebar
+        if (screen->windowMenuFilename().empty() ||
+            ! createWindowMenuFromFile(screen->windowMenuFilename(), *menu, true)) {
+            char default_menu[][11] = {
+                "shade", 
+                "stick",
+                "maximize",
+                "iconify",
+                "raise",
+                "lower",
+                "sendto",
+                "layer",
+                "extramenus",
+                "separator",
+                "close",
+                0
+            };
+            for (int i=0; i < sizeof(default_menu); ++i)
+                createWindowMenuItem(default_menu[i], "", *menu);
+        }
+        menu->reconfigure(); // update graphics
+        return menu;
     }
+
     return 0;
 }
 
-bool MenuCreator::createWindowMenuItem(const std::string &type, 
+bool MenuCreator::createWindowMenuItem(const std::string &type,
                                        const std::string &label,
-                                       FbTk::Menu &menu,
-                                       FluxboxWindow &win) {
+                                       FbTk::Menu &menu) {
     typedef FbTk::RefCount<FbTk::Command> RefCmd;
-    typedef FbTk::SimpleCommand<FluxboxWindow> WindowCmd;
     _FB_USES_NLS;
 
     if (type == "shade") {
-        RefCmd shade_cmd(new WindowCmd(win, &FluxboxWindow::shade));
+        RefCmd shade_cmd(new WindowCmd<void>(&FluxboxWindow::shade));
         menu.insert(label.empty()?_FBTEXT(Windowmenu, Shade, "Shade", "Shade the window"):label.c_str(), shade_cmd);
     } else if (type == "maximize") {
-        RefCmd maximize_cmd(new WindowCmd(win, &FluxboxWindow::maximizeFull));
-        RefCmd maximize_vert_cmd(new WindowCmd(win, &FluxboxWindow::maximizeVertical));
-        RefCmd maximize_horiz_cmd(new WindowCmd(win, &FluxboxWindow::maximizeHorizontal));
-        FbTk::MultiButtonMenuItem *maximize_item = new FbTk::MultiButtonMenuItem(3, label.empty()?_FBTEXT(Windowmenu, Maximize, "Maximize", "Maximize the window"):label.c_str());
+        RefCmd maximize_cmd(new WindowCmd<void>(&FluxboxWindow::maximizeFull));
+        RefCmd maximize_vert_cmd(new WindowCmd<void>(&FluxboxWindow::maximizeVertical));
+        RefCmd maximize_horiz_cmd(new WindowCmd<void>(&FluxboxWindow::maximizeHorizontal));
+        FbTk::MultiButtonMenuItem *maximize_item = 
+            new FbTk::MultiButtonMenuItem(3, 
+                                          label.empty()?
+                                          _FBTEXT(Windowmenu, Maximize, 
+                                                  "Maximize", "Maximize the window"):
+                                          label.c_str());
         // create maximize item with:
         // button1: Maximize normal
         // button2: Maximize Vertical
@@ -471,43 +510,49 @@ bool MenuCreator::createWindowMenuItem(const std::string &type,
         maximize_item->setCommand(3, maximize_horiz_cmd);
         menu.insert(maximize_item);
     } else if (type == "iconify") {
-        RefCmd iconify_cmd(new WindowCmd(win, &FluxboxWindow::iconify));
+        RefCmd iconify_cmd(new WindowCmd<void>(&FluxboxWindow::iconify));
         menu.insert(label.empty()?_FBTEXT(Windowmenu, Iconify, "Iconify", "Iconify the window"):label.c_str(), iconify_cmd);
     } else if (type == "close") {
-        RefCmd close_cmd(new WindowCmd(win, &FluxboxWindow::close));
+        RefCmd close_cmd(new WindowCmd<void>(&FluxboxWindow::close));
         menu.insert(label.empty()?_FBTEXT(Windowmenu, Close, "Close", "Close the window"):label.c_str(), close_cmd);
     } else if (type == "kill" || type == "killwindow") {
-        RefCmd kill_cmd(new WindowCmd(win, &FluxboxWindow::kill));
+        RefCmd kill_cmd(new WindowCmd<void>(&FluxboxWindow::kill));
         menu.insert(label.empty()?_FBTEXT(Windowmenu, Kill, "Kill", "Kill the window"):label.c_str(), kill_cmd);
     } else if (type == "lower") {
-        RefCmd lower_cmd(new WindowCmd(win, &FluxboxWindow::lower));
+        RefCmd lower_cmd(new WindowCmd<void>(&FluxboxWindow::lower));
         menu.insert(label.empty()?_FBTEXT(Windowmenu, Lower, "Lower", "Lower the window"):label.c_str(), lower_cmd);
     } else if (type == "raise") {
-        RefCmd raise_cmd(new WindowCmd(win, &FluxboxWindow::raise));
+        RefCmd raise_cmd(new WindowCmd<void>(&FluxboxWindow::raise));
         menu.insert(label.empty()?_FBTEXT(Windowmenu, Raise, "Raise", "Raise the window"):label.c_str(), raise_cmd);
     } else if (type == "stick") {
-        RefCmd stick_cmd(new WindowCmd(win, &FluxboxWindow::stick));
+        RefCmd stick_cmd(new WindowCmd<void>(&FluxboxWindow::stick));
         menu.insert(label.empty()?_FBTEXT(Windowmenu, Stick, "Stick", "Stick the window"):label.c_str(), stick_cmd);
-    } else if (type == "extramenus") {
-        FluxboxWindow::ExtraMenus::iterator it = win.extraMenus().begin();
-        FluxboxWindow::ExtraMenus::iterator it_end = win.extraMenus().end();
+    } 
+    else if (type == "extramenus") {
+        BScreen *screen = Fluxbox::instance()->findScreen(menu.screenNumber());
+        BScreen::ExtraMenus::iterator it = screen->extraWindowMenus().begin();
+        BScreen::ExtraMenus::iterator it_end = screen->extraWindowMenus().end();
         for (; it != it_end; ++it) {
             it->second->disableTitle();
             menu.insert(it->first, it->second);
         }
-
+        
     } else if (type == "sendto") {
-        menu.insert(label.empty()?_FBTEXT(Windowmenu, SendTo, "Send To...", "Send to menu item name"):label.c_str(), new SendToMenu(win));
-    } else if (type == "layer") {
+        menu.insert(label.empty() ? _FBTEXT(Windowmenu, SendTo, "Send To...", "Send to menu item name"):
+                    label.c_str(), new SendToMenu(*Fluxbox::instance()->findScreen(menu.screenNumber())));
+    }else if (type == "layer") {
         BScreen *screen = Fluxbox::instance()->findScreen(menu.screenNumber());
         if (screen == 0)
             return false;
-        FbTk::Menu *submenu = new LayerMenu<FluxboxWindow>(screen->menuTheme(), 
-                                                           screen->imageControl(), 
-                                                           *screen->layerManager().
-                                                           getLayer(Fluxbox::instance()->getMenuLayer()), 
-                                                           &win,
-                                                           false);
+
+        static MenuContext context;
+
+        FbTk::Menu *submenu = new LayerMenu(screen->menuTheme(),
+                                            screen->imageControl(),
+                                            *screen->layerManager().
+                                            getLayer(Fluxbox::instance()->getMenuLayer()),
+                                            &context,
+                                            false);
         submenu->disableTitle();
         menu.insert(label.empty()?_FBTEXT(Windowmenu, Layer, "Layer ...", "Layer menu"):label.c_str(), submenu);
 
