@@ -1,8 +1,8 @@
 // Remember.cc for Fluxbox Window Manager
+// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+//                     and Simon Bowden    (rathnor at users.sourceforge.net)
 // Copyright (c) 2002 Xavier Brouckaert
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
-//                and Simon Bowden    (rathnor at users.sourceforge.net)
-//
+// 
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
 // to deal in the Software without restriction, including without limitation
@@ -21,7 +21,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Remember.cc 3700 2004-11-19 11:37:27Z mathias $
+// $Id: Remember.cc 4066 2005-06-23 09:22:11Z mathias $
 
 #include "Remember.hh"
 #include "ClientPattern.hh"
@@ -31,11 +31,13 @@
 #include "FbMenu.hh"
 #include "FbCommands.hh"
 #include "fluxbox.hh"
+#include "WindowCmd.hh"
 
 #include "FbTk/I18n.hh"
 #include "FbTk/StringUtil.hh"
 #include "FbTk/MenuItem.hh"
 #include "FbTk/App.hh"
+#include "FbTk/stringstream.hh"
 
 
 #include <X11/Xlib.h>
@@ -52,19 +54,6 @@
 #include <memory>
 #include <set>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif // HAVE_CONFIG_H
-
-#ifdef HAVE_SSTREAM
-#include <sstream>
-#define FB_istringstream istringstream
-#elif HAVE_STRSTREAM 
-#include <strstream>
-#define FB_istringstream istrstream
-#else
-#error "You dont have sstream or strstream headers!"
-#endif // HAVE_STRSTREAM
 
 using namespace std;
 
@@ -72,82 +61,89 @@ namespace {
 
 class RememberMenuItem : public FbTk::MenuItem {
 public:
-    RememberMenuItem(const char *label, Remember &remember,
-                     FluxboxWindow &fbwin,
+    RememberMenuItem(const char *label,
                      Remember::Attribute attrib) :
-        FbTk::MenuItem(label), m_remember(remember), 
-        m_win(fbwin), m_attrib(attrib) {
-        setToggleItem(true);        
+        FbTk::MenuItem(label),
+        m_attrib(attrib) {
+        setToggleItem(true);
     }
 
     bool isSelected() const {
-        if (m_win.numClients()) // ensure it HAS clients
-            return m_remember.isRemembered(m_win.winClient(), m_attrib);
+        if (WindowCmd<void>::window() == 0)
+            return false;
+
+        if (WindowCmd<void>::window()->numClients()) // ensure it HAS clients
+            return Remember::instance().isRemembered(WindowCmd<void>::window()->winClient(), m_attrib);
         else
             return false;
     }
 
     bool isEnabled() const {
-        if (m_attrib != Remember::REM_JUMPWORKSPACE) 
+        if (WindowCmd<void>::window() == 0)
+            return false;
+
+        if (m_attrib != Remember::REM_JUMPWORKSPACE)
             return true;
-        else if (m_win.numClients())
-            return (m_remember.isRemembered(m_win.winClient(), Remember::REM_WORKSPACE));
+        else if (WindowCmd<void>::window()->numClients())
+            return (Remember::instance().isRemembered(WindowCmd<void>::window()->winClient(), Remember::REM_WORKSPACE));
         else
             return false;
     }
 
     void click(int button, int time) {
-        if (isSelected()) {
-            m_remember.forgetAttrib(m_win.winClient(), m_attrib);
-        } else {
-            m_remember.rememberAttrib(m_win.winClient(), m_attrib);
+        if (WindowCmd<void>::window() != 0) {
+            if (isSelected()) {
+                Remember::instance().forgetAttrib(WindowCmd<void>::window()->winClient(), m_attrib);
+            } else {
+                Remember::instance().rememberAttrib(WindowCmd<void>::window()->winClient(), m_attrib);
+            }
         }
-        m_remember.save();
+        Remember::instance().save();
         FbTk::MenuItem::click(button, time);
     }
 
 private:
-    // my remember manager
-    Remember &m_remember;
-    FluxboxWindow &m_win;
     Remember::Attribute m_attrib;
 };
 
-FbTk::Menu *createRememberMenu(Remember &remember, FluxboxWindow &win, bool enabled) {
+FbTk::Menu *createRememberMenu(BScreen &screen) {
     // each fluxboxwindow has its own windowmenu
     // so we also create a remember menu just for it...
-    FbTk::Menu *menu = win.screen().createMenu("");
+    FbTk::Menu *menu = screen.createMenu("");
 
     // if enabled, then we want this to be a unavailable menu
+    /*
     if (!enabled) {
         FbTk::MenuItem *item = new FbTk::MenuItem("unavailable");
         item->setEnabled(false);
         menu->insert(item);
-        menu->update();
+        menu->updateMenu();
         return menu;
     }
-    
+    */
     _FB_USES_NLS;
-    menu->insert(new RememberMenuItem(_FBTEXT(Remember, Workspace, "Workspace", "Remember Workspace"), 
-                                      remember, win, Remember::REM_WORKSPACE));
+    menu->insert(new RememberMenuItem(_FBTEXT(Remember, Workspace, "Workspace", "Remember Workspace"),
+                                      Remember::REM_WORKSPACE));
     menu->insert(new RememberMenuItem(_FBTEXT(Remember, JumpToWorkspace, "Jump to workspace", "Change active workspace to remembered one on open"),
-                                      remember, win, Remember::REM_JUMPWORKSPACE));
-    menu->insert(new RememberMenuItem(_FBTEXT(Remember, Dimensions, "Dimensions", "Remember Dimensions - windth width and height"),
-                                      remember, win, Remember::REM_DIMENSIONS));
-    menu->insert(new RememberMenuItem(_FBTEXT(Remember, Position, "Position", "Remember position - window co-ordinates"), 
-                                      remember, win, Remember::REM_POSITION));
-    menu->insert(new RememberMenuItem(_FBTEXT(Remember, Sticky, "Sticky", "Remember Sticky"), 
-                                      remember, win, Remember::REM_STUCKSTATE));
+                                      Remember::REM_JUMPWORKSPACE));
+    menu->insert(new RememberMenuItem(_FBTEXT(Remember, Head, "Head", "Remember Head"),
+                                      Remember::REM_HEAD));
+    menu->insert(new RememberMenuItem(_FBTEXT(Remember, Dimensions, "Dimensions", "Remember Dimensions - with width and height"),
+                                      Remember::REM_DIMENSIONS));
+    menu->insert(new RememberMenuItem(_FBTEXT(Remember, Position, "Position", "Remember position - window co-ordinates"),
+                                      Remember::REM_POSITION));
+    menu->insert(new RememberMenuItem(_FBTEXT(Remember, Sticky, "Sticky", "Remember Sticky"),
+                                      Remember::REM_STUCKSTATE));
     menu->insert(new RememberMenuItem(_FBTEXT(Remember, Decorations, "Decorations", "Remember window decorations"),
-                                      remember, win, Remember::REM_DECOSTATE));
+                                      Remember::REM_DECOSTATE));
     menu->insert(new RememberMenuItem(_FBTEXT(Remember, Shaded, "Shaded", "Remember shaded"),
-                                      remember, win, Remember::REM_SHADEDSTATE));
+                                      Remember::REM_SHADEDSTATE));
     menu->insert(new RememberMenuItem(_FBTEXT(Remember, Layer, "Layer", "Remember Layer"),
-                                      remember, win, Remember::REM_LAYER));
+                                      Remember::REM_LAYER));
     menu->insert(new RememberMenuItem(_FBTEXT(Remember, SaveOnClose, "Save on close", "Save remembered attributes on close"),
-                                      remember, win, Remember::REM_SAVEONCLOSE));
+                                      Remember::REM_SAVEONCLOSE));
 
-    menu->update();
+    menu->updateMenu();
     return menu;
 };
 
@@ -160,8 +156,8 @@ bool handleStartupItem(const string &line, int offset) {
 
     // accept some options, for now only "screen=NN"
     // these option are given in parentheses before the command
-    next = FbTk::StringUtil::getStringBetween(str, 
-                                              line.c_str() + offset, 
+    next = FbTk::StringUtil::getStringBetween(str,
+                                              line.c_str() + offset,
                                               '(', ')');
     if (next > 0) {
         // there are some options
@@ -171,7 +167,7 @@ bool handleStartupItem(const string &line, int offset) {
         if (pos > 0) {
             option = str.substr(0, pos);
             if (option == "screen") {
-                FB_istringstream iss(str.c_str() + pos + 1);
+                FbTk_istringstream iss(str.c_str() + pos + 1);
                 iss >> screen;
             } else {
                 error = true;
@@ -187,8 +183,8 @@ bool handleStartupItem(const string &line, int offset) {
         next = 0;
     }
 
-    next = FbTk::StringUtil::getStringBetween(str, 
-                                              line.c_str() + offset + next, 
+    next = FbTk::StringUtil::getStringBetween(str,
+                                              line.c_str() + offset + next,
                                               '{', '}');
 
     if (next <= 0) {
@@ -212,30 +208,38 @@ Application::Application(bool grouped)
     : is_grouped(grouped),
       group(0)
 {
-    decostate_remember = 
+    decostate_remember =
         dimensions_remember =
-        focushiddenstate_remember = 
+        focushiddenstate_remember =
         iconhiddenstate_remember =
-        jumpworkspace_remember = 
-        layer_remember  = 
+        jumpworkspace_remember =
+        layer_remember  =
         position_remember =
         shadedstate_remember =
-        stuckstate_remember = 
+        stuckstate_remember =
         tabstate_remember =
-        workspace_remember = 
+        workspace_remember =
+        head_remember =
         save_on_close_remember = false;
 }
 
-/********************************************************
+/************
  * Remember *
  ************/
 
+Remember *Remember::s_instance = 0;
+
 Remember::Remember() {
+    if (s_instance != 0)
+        throw string("Can not create more than one instance of Remember");
+
+    s_instance = this;
     enableUpdate();
     load();
 }
 
 Remember::~Remember() {
+
     // free our resources
 
     // the patterns free the "Application"s
@@ -254,6 +258,8 @@ Remember::~Remember() {
         delete (*ait);
         ++ait;
     }
+
+    s_instance = 0;
 }
 
 Application* Remember::find(WinClient &winclient) {
@@ -264,7 +270,7 @@ Application* Remember::find(WinClient &winclient) {
         return wc_it->second;
     else {
         Patterns::iterator it = m_pats.begin();
-        for (; it != m_pats.end(); it++) 
+        for (; it != m_pats.end(); it++)
             if (it->first->match(winclient)) {
                 it->first->addMatch();
                 m_clients[&winclient] = it->second;
@@ -304,8 +310,8 @@ int Remember::parseApp(ifstream &file, Application &app, string *first_line) {
                 continue;  //the line is commented or blank
             int parse_pos = 0, err = 0;
             string str_key, str_option, str_label;
-            err = FbTk::StringUtil::getStringBetween(str_key, 
-                                                     line.c_str(), 
+            err = FbTk::StringUtil::getStringBetween(str_key,
+                                                     line.c_str(),
                                                      '[', ']');
             if (err > 0) {
                 int tmp;
@@ -317,8 +323,8 @@ int Remember::parseApp(ifstream &file, Application &app, string *first_line) {
             }
             if (err > 0 ) {
                 parse_pos += err;
-                err = FbTk::StringUtil::getStringBetween(str_label, 
-                                                         line.c_str() + parse_pos, 
+                err = FbTk::StringUtil::getStringBetween(str_label,
+                                                         line.c_str() + parse_pos,
                                                          '{', '}');
                 if (err>0) {
                     parse_pos += err;
@@ -330,9 +336,12 @@ int Remember::parseApp(ifstream &file, Application &app, string *first_line) {
                 continue; //read next line
             if (str_key == "Workspace") {
                 unsigned int w;
-                FB_istringstream iss(str_label.c_str());
+                FbTk_istringstream iss(str_label.c_str());
                 iss >> w;
                 app.rememberWorkspace(w);
+            } else if (str_key == "Head") {
+                int h = atoi(str_label.c_str());
+                app.rememberHead(h);
             } else if (str_key == "Layer") {
                 unsigned int l;
                 if (str_label == "DESKTOP") {
@@ -350,23 +359,22 @@ int Remember::parseApp(ifstream &file, Application &app, string *first_line) {
                 } else if (str_label == "MENU") {
                     l = Fluxbox::instance()->getMenuLayer();
                 } else {
-                    FB_istringstream iss(str_label.c_str());
+                    FbTk_istringstream iss(str_label.c_str());
                     iss >> l;
                 }
                 app.rememberLayer(l);
             } else if (str_key == "Dimensions") {
                 unsigned int h,w;
-                FB_istringstream iss(str_label.c_str());
+                FbTk_istringstream iss(str_label.c_str());
                 iss >> w >> h;
                 app.rememberDimensions(w,h);
             } else if (str_key == "Position") {
-                FB_istringstream iss;
                 unsigned int r= 0;
                 unsigned int x= 0;
                 unsigned int y= 0;
                 // more info about the parameter
                 // in ::rememberPosition
-                
+
                 if ( str_option.length() )
                     {
                         if      ( str_option == "UPPERLEFT"  ) r= POS_UPPERLEFT;
@@ -376,22 +384,22 @@ int Remember::parseApp(ifstream &file, Application &app, string *first_line) {
                         else if ( str_option == "CENTER" )     r= POS_CENTER;
                         else if ( str_option == "WINCENTER" )  r= POS_WINCENTER;
                         else {
-                            iss.str(str_option);
-                            iss >> r;
+                            FbTk_istringstream iss_r(str_option.c_str());
+                            iss_r >> r;
                         }
                     }
 
-                iss.str(str_label.c_str());
-                iss >> x >> y;
+                FbTk_istringstream iss_xy(str_label.c_str());
+                iss_xy >> x >> y;
                 app.rememberPosition(x, y, r);
             } else if (str_key == "Shaded") {
                 app.rememberShadedstate((str_label=="yes"));
             } else if (str_key == "Tab") {
                 app.rememberTabstate((str_label=="yes"));
             } else if (str_key == "FocusHidden") {
-                app.rememberFocusHiddenstate((str_label=="yes")); 
+                app.rememberFocusHiddenstate((str_label=="yes"));
             } else if (str_key == "IconHidden") {
-                app.rememberIconHiddenstate((str_label=="yes")); 
+                app.rememberIconHiddenstate((str_label=="yes"));
             } else if (str_key == "Hidden") {
                 app.rememberIconHiddenstate((str_label=="yes"));
                 app.rememberFocusHiddenstate((str_label=="yes"));
@@ -402,7 +410,7 @@ int Remember::parseApp(ifstream &file, Application &app, string *first_line) {
                     app.rememberDecostate((unsigned int) 0xfffffff);
                 } else if (str_label == "TINY") {
                     app.rememberDecostate((unsigned int)
-                                          FluxboxWindow::DECORM_TITLEBAR 
+                                          FluxboxWindow::DECORM_TITLEBAR
                                           | FluxboxWindow::DECORM_ICONIFY
                                           | FluxboxWindow::DECORM_MENU
                                           );
@@ -420,7 +428,7 @@ int Remember::parseApp(ifstream &file, Application &app, string *first_line) {
                     unsigned int mask;
                     const char * str = str_label.c_str();
                     // it'll have at least one char and \0, so this is safe
-                    FB_istringstream iss(str);
+                    FbTk_istringstream iss(str);
                     // check for hex
                     if (str[0] == '0' && str[1] == 'x') {
                         iss.seekg(2);
@@ -468,8 +476,8 @@ void Remember::load() {
                     continue;
                 string key;
                 int err=0;
-                int pos = FbTk::StringUtil::getStringBetween(key, 
-                                                             line.c_str(), 
+                int pos = FbTk::StringUtil::getStringBetween(key,
+                                                             line.c_str(),
                                                              '[', ']');
 
                 if (pos > 0 && key == "app") {
@@ -502,7 +510,7 @@ void Remember::load() {
                         m_pats.push_back(make_pair(grouped_pats.front(), app));
                         grouped_pats.pop_front();
                     }
-                    
+
                     // we hit end... probably don't have attribs for the group
                     // so finish it off with an empty application
                     // otherwise parse the app
@@ -512,7 +520,7 @@ void Remember::load() {
                     in_group = false;
                 } else
                     cerr<<"Error in apps file on line "<<row<<"."<<endl;
-                
+
             }
         } else {
 #ifdef DEBUG
@@ -566,11 +574,34 @@ void Remember::save() {
         if (a.workspace_remember) {
             apps_file << "  [Workspace]\t{" << a.workspace << "}" << endl;
         }
+        if (a.head_remember) {
+            apps_file << "  [Head]\t{" << a.head << "}" << endl;
+        }
         if (a.dimensions_remember) {
             apps_file << "  [Dimensions]\t{" << a.w << " " << a.h << "}" << endl;
         }
         if (a.position_remember) {
-            apps_file << "  [Position]\t{" << a.x << " " << a.y << "}" << endl;
+            apps_file << "  [Position]\t(";
+            switch(a.refc) {
+            case POS_WINCENTER:
+                apps_file << "WINCENTER";
+                break;
+            case POS_CENTER:
+                apps_file << "CENTER";
+                break;
+            case POS_LOWERLEFT:
+                apps_file << "LOWERLEFT";
+                break;
+            case POS_LOWERRIGHT:
+                apps_file << "LOWERRIGHT";
+                break;
+            case POS_UPPERRIGHT:
+                apps_file << "UPPERRIGHT";
+                break;
+            default:
+                apps_file << "UPPERLEFT";
+            }
+            apps_file << ")\t{" << a.x << " " << a.y << "}" << endl;
         }
         if (a.shadedstate_remember) {
             apps_file << "  [Shaded]\t{" << ((a.shadedstate)?"yes":"no") << "}" << endl;
@@ -581,22 +612,22 @@ void Remember::save() {
         if (a.decostate_remember) {
             switch (a.decostate) {
             case (0) :
-                apps_file << "  [Deco]\t{NONE}" << endl; 
+                apps_file << "  [Deco]\t{NONE}" << endl;
                 break;
             case (0xffffffff):
             case (FluxboxWindow::DECORM_LAST - 1):
                 apps_file << "  [Deco]\t{NORMAL}" << endl;
                 break;
-            case (FluxboxWindow::DECORM_TITLEBAR 
+            case (FluxboxWindow::DECORM_TITLEBAR
                   | FluxboxWindow::DECORM_ICONIFY
                   | FluxboxWindow::DECORM_MENU):
                 apps_file << "  [Deco]\t{TOOL}" << endl;
                 break;
-            case (FluxboxWindow::DECORM_TITLEBAR 
+            case (FluxboxWindow::DECORM_TITLEBAR
                   | FluxboxWindow::DECORM_MENU):
                 apps_file << "  [Deco]\t{TINY}" << endl;
                 break;
-            case (FluxboxWindow::DECORM_BORDER 
+            case (FluxboxWindow::DECORM_BORDER
                   | FluxboxWindow::DECORM_MENU):
                 apps_file << "  [Deco]\t{BORDER}" << endl;
                 break;
@@ -608,7 +639,7 @@ void Remember::save() {
 
         if (a.focushiddenstate_remember || a.iconhiddenstate_remember) {
             if (a.focushiddenstate_remember && a.iconhiddenstate_remember &&
-                a.focushiddenstate && a.iconhiddenstate) 
+                a.focushiddenstate && a.iconhiddenstate)
                 apps_file << "  [Hidden]\t{" << ((a.focushiddenstate)?"yes":"no") << "}" << endl;
             else if (a.focushiddenstate_remember) {
                 apps_file << "  [FocusHidden]\t{" << ((a.focushiddenstate)?"yes":"no") << "}" << endl;
@@ -639,6 +670,9 @@ bool Remember::isRemembered(WinClient &winclient, Attribute attrib) {
     case REM_WORKSPACE:
         return app->workspace_remember;
         break;
+    case REM_HEAD:
+        return app->head_remember;
+        break;
     case REM_DIMENSIONS:
         return app->dimensions_remember;
         break;
@@ -690,6 +724,9 @@ void Remember::rememberAttrib(WinClient &winclient, Attribute attrib) {
     case REM_WORKSPACE:
         app->rememberWorkspace(win->workspaceNumber());
         break;
+    case REM_HEAD:
+        app->rememberHead(win->screen().getHead(win->fbWindow()));
+        break;
     case REM_DIMENSIONS:
         app->rememberDimensions(win->width(), win->height());
         break;
@@ -741,6 +778,9 @@ void Remember::forgetAttrib(WinClient &winclient, Attribute attrib) {
     case REM_WORKSPACE:
         app->forgetWorkspace();
         break;
+    case REM_HEAD:
+        app->forgetHead();
+        break;
     case REM_DIMENSIONS:
         app->forgetDimensions();
         break;
@@ -786,15 +826,12 @@ void Remember::setupFrame(FluxboxWindow &win) {
     // we don't touch the window if it is a transient
     // of something else
 
-    // All windows get the remember menu.
-    win.addExtraMenu(_FBTEXT(Remember, MenuItemName, "Remember...", "Remember item in menu"),
-                     createRememberMenu(*this, win, (winclient.transientFor() == 0)));
 
-    if (winclient.transientFor()) 
+    if (winclient.transientFor())
         return;
 
     Application *app = find(winclient);
-    if (app == 0) 
+    if (app == 0)
         return; // nothing to do
 
     if (app->is_grouped && app->group == 0)
@@ -803,42 +840,52 @@ void Remember::setupFrame(FluxboxWindow &win) {
     BScreen &screen = winclient.screen();
 
     if (app->workspace_remember) {
-        // TODO: fix placement to initialise properly
-        screen.reassociateWindow(&win, app->workspace, true); 
+        // we use setWorkspace and not reassoc because we're still initialising
+        win.setWorkspace(app->workspace);
         if (app->jumpworkspace_remember)
             screen.changeWorkspaceID(app->workspace);
     }
 
+    if (app->head_remember) {
+        win.screen().setOnHead<FluxboxWindow>(win, app->head);
+    }
+
     if (app->decostate_remember)
         win.setDecorationMask(app->decostate);
 
 
     if (app->dimensions_remember)
         win.resize(app->w, app->h);
-    
+
+    int head = screen.getHead(win.fbWindow());
+
     if (app->position_remember) {
+
         switch (app->refc) {
           default:
           case POS_UPPERLEFT: // upperleft corner
-        win.move(app->x, app->y);
+              win.move(screen.getHeadX(head) + app->x,
+                       screen.getHeadY(head) + app->y);
               break;
           case POS_UPPERRIGHT: // upperright corner
-              win.move(screen.width() - win.width() - app->x, app->y);
+              win.move(screen.getHeadX(head) + screen.getHeadWidth(head) - win.width() - app->x,
+                       screen.getHeadY(head) + app->y);
               break;
           case POS_LOWERLEFT: // lowerleft corner
-              win.move(app->x, screen.height() - win.height() - app->y);
+              win.move(screen.getHeadX(head) + app->x,
+                       screen.getHeadHeight(head) - win.height() - app->y);
               break;
           case POS_LOWERRIGHT: // lowerright corner
-              win.move(screen.width() - win.width() - app->x,
-                       screen.height() - win.height() - app->y);
+              win.move(screen.getHeadWidth(head) - win.width() - app->x,
+                       screen.getHeadHeight(head) - win.height() - app->y);
               break;
           case POS_CENTER: // center of the screen, windows topleft corner is on the center
-                win.move((screen.width() / 2) + app->x,
-                         (screen.height() / 2) + app->y);
+                win.move((screen.getHeadWidth(head) / 2) + app->x,
+                         (screen.getHeadHeight(head) / 2) + app->y);
               break;
           case POS_WINCENTER: // the window is centered REALLY upon the center
-                win.move((screen.width() / 2) - ( win.width() / 2 ) + app->x,
-                         (screen.height() / 2) - ( win.height() / 2 ) + app->y);
+                win.move((screen.getHeadWidth(head) / 2) - ( win.width() / 2 ) + app->x,
+                         (screen.getHeadHeight(head) / 2) - ( win.height() / 2 ) + app->y);
               break;
         };
     }
@@ -870,7 +917,7 @@ void Remember::setupFrame(FluxboxWindow &win) {
 void Remember::setupClient(WinClient &winclient) {
 
     Application *app = find(winclient);
-    if (app == 0) 
+    if (app == 0)
         return; // nothing to do
 
     if (winclient.fbwindow() == 0 && app->is_grouped && app->group) {
@@ -901,6 +948,14 @@ void Remember::updateClientClose(WinClient &winclient) {
 
 }
 
+void Remember::initForScreen(BScreen &screen) {
+    // All windows get the remember menu.
+    _FB_USES_NLS;
+    screen.addExtraWindowMenu(_FBTEXT(Remember, MenuItemName, "Remember...", "Remember item in menu"),
+                              createRememberMenu(screen));
+
+}
+
 void Remember::updateFrameClose(FluxboxWindow &win) {
     // scan all applications and remove this fbw if it is a recorded group
     Patterns::iterator it = m_pats.begin();
