@@ -1,5 +1,5 @@
 // Screen.cc for Fluxbox Window Manager
-// Copyright (c) 2001 - 2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2001 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Screen.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Screen.cc 3736 2004-11-30 22:19:35Z mathias $
+// $Id: Screen.cc 4099 2005-09-07 16:46:17Z simonb $
 
 
 #include "Screen.hh"
@@ -69,6 +69,8 @@
 #include "FbTk/ImageControl.hh"
 #include "FbTk/EventManager.hh"
 #include "FbTk/Transparent.hh"
+#include "FbTk/Select2nd.hh"
+#include "FbTk/Compose.hh"
 
 //use GNU extensions
 #ifndef	 _GNU_SOURCE
@@ -174,7 +176,8 @@ BScreen::ScreenResource::ScreenResource(FbTk::ResourceManager &rm,
     rootcommand(rm, "", scrname+".rootCommand", altscrname+".RootCommand"),
     resize_model(rm, BOTTOMRESIZE, scrname+".resizeMode", altscrname+".ResizeMode"),
     windowmenufile(rm, "", scrname+".windowMenu", altscrname+".WindowMenu"),
-    focus_model(rm, CLICKTOFOCUS, scrname+".focusModel", altscrname+".FocusModel"),
+    focus_model(rm, CLICKFOCUS, scrname+".focusModel", altscrname+".FocusModel"),
+    tabfocus_model(rm, CLICKTABFOCUS, scrname+".tabFocusModel", altscrname+".TabFocusModel"),
     follow_model(rm, IGNORE_OTHER_WORKSPACES, scrname+".followModel", altscrname+".followModel"),
     workspaces(rm, 1, scrname+".workspaces", altscrname+".Workspaces"),
     edge_snap_threshold(rm, 0, scrname+".edgeSnapThreshold", altscrname+".EdgeSnapThreshold"),
@@ -199,7 +202,10 @@ BScreen::ScreenResource::ScreenResource(FbTk::ResourceManager &rm,
     gc_cap_style(rm,
                  FbTk::GContext::CAPNOTLAST,
                  scrname+".overlay.capStyle",
-                 altscrname+".overlay.CapStyle") {
+                 altscrname+".overlay.CapStyle"),
+    scroll_action(rm, "", scrname+".windowScrollAction", altscrname+".WindowScrollAction"),
+    scroll_reverse(rm, false, scrname+".windowScrollReverse", altscrname+".WindowScrollReverse") {
+    
 
 }
 
@@ -308,14 +314,6 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
 
     imageControl().setDither(*resource.image_dither);
 
-    // setup windowtheme for antialias
-    // before we load the theme
-
-    winFrameTheme().font().setAntialias(*resource.antialias);
-    menuTheme().titleFont().setAntialias(*resource.antialias);
-    menuTheme().frameFont().setAntialias(*resource.antialias);
-
-
     winFrameTheme().reconfigSig().attach(this);// for geom window
 
 
@@ -328,22 +326,12 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
     renderPosWindow();
 
     // setup workspaces and workspace menu
-
-    if (*resource.workspaces != 0) {
-        for (int i = 0; i < *resource.workspaces; ++i) {
-            Workspace *wkspc = new Workspace(*this, m_layermanager, 
-                                             getNameOfWorkspace(m_workspaces_list.size()),
-                                             m_workspaces_list.size());
-            m_workspaces_list.push_back(wkspc);
-        }
-    } else { // create at least one workspace
-        Workspace *wkspc = new Workspace(*this, m_layermanager, 
-                                         getNameOfWorkspace(m_workspaces_list.size()),
-                                         m_workspaces_list.size());
-        m_workspaces_list.push_back(wkspc);
+    int nr_ws = *resource.workspaces;
+    addWorkspace(); // at least one
+    for (int i = 1; i < nr_ws; ++i) {
+        addWorkspace();
     }
 
-
     m_current_workspace = m_workspaces_list.front();
 
 
@@ -351,7 +339,8 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
     // own resources we must do this.
     fluxbox->load_rc(*this);
 
-    m_configmenu.reset(createMenu(_FBTEXT(Menu, Configuration, "Configuration", "Title of configuration menu")));
+    m_configmenu.reset(createMenu(_FBTEXT(Menu, Configuration, 
+                                  "Configuration", "Title of configuration menu")));
     setupConfigmenu(*m_configmenu.get());
     m_configmenu->setInternalMenu();
 
@@ -368,7 +357,8 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
     // So we lock root theme temporary so it doesn't uses RootTheme::reconfigTheme
     // This must be fixed in the future.
     m_root_theme->lock(true);
-    FbTk::ThemeManager::instance().load(fluxbox->getStyleFilename());
+    FbTk::ThemeManager::instance().load(fluxbox->getStyleFilename(),
+                                        m_root_theme->screenNum());
     m_root_theme->lock(false);
     m_root_theme->setLineAttributes(*resource.gc_line_width,
                                     *resource.gc_line_style,
@@ -381,7 +371,7 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
 #endif // SLIT
 
     rm.unlock();
-
+  
     XFlush(disp);
 }
 
@@ -398,6 +388,7 @@ void destroyAndClearList(A &a) {
 
 BScreen::~BScreen() {
     
+
     if (! managed)
         return;
     
@@ -408,6 +399,24 @@ BScreen::~BScreen() {
     // we need to destroy it before we destroy workspaces
     m_workspacemenu.reset(0);
 
+    ExtraMenus::iterator mit = m_extramenus.begin();
+    ExtraMenus::iterator mit_end = m_extramenus.end();
+    for (; mit != mit_end; ++mit) {
+        // we set them to NOT internal so that they will be deleted when the
+        // menu is cleaned up. We can't delete them here because they are
+        // still in the menu
+        // (They need to be internal for most of the time so that if we
+        // rebuild the menu, then they won't be removed.
+        if (mit->second->parent() == 0) {
+            // not attached to our windowmenu
+            // so we clean it up
+            delete mit->second;
+        } else {
+            // let the parent clean it up
+            mit->second->setInternalMenu(false);
+        }
+    }
+    
     if (geom_pixmap != None)
         imageControl().removeImage(geom_pixmap);
 
@@ -615,6 +624,25 @@ FbTk::Menu *BScreen::createMenu(const std::string &label) {
     return menu;
 }
 
+void BScreen::addExtraWindowMenu(const char *label, FbTk::Menu *menu) {
+    menu->setInternalMenu();
+    menu->disableTitle();
+    m_extramenus.push_back(std::make_pair(label, menu));
+    // recreate window menu
+    m_windowmenu.reset(MenuCreator::createMenuType("windowmenu", screenNumber()));
+}
+
+void BScreen::removeExtraWindowMenu(FbTk::Menu *menu) {
+    ExtraMenus::iterator it = find_if(m_extramenus.begin(),
+                                      m_extramenus.end(),
+                                      FbTk::Compose(bind2nd(equal_to<FbTk::Menu *>(), menu),
+                                                    FbTk::Select2nd<ExtraMenus::value_type>()));
+    if (it != m_extramenus.end())
+        m_extramenus.erase(it);
+    // recreate window menu
+    m_windowmenu.reset(MenuCreator::createMenuType("windowmenu", screenNumber()));
+}
+
 void BScreen::hideMenus() {
     // hide extra menus
     Fluxbox::instance()->hideExtraMenus(*this);
@@ -626,9 +654,9 @@ void BScreen::hideMenus() {
 #endif // SLIT
 
     // hide icon menus
-    if (getIconList().size()) {
-        Icons::iterator it = getIconList().begin();
-        const Icons::iterator it_end = getIconList().end();
+    if (iconList().size()) {
+        Icons::iterator it = iconList().begin();
+        const Icons::iterator it_end = iconList().end();
         for (; it != it_end; ++it)
             (*it)->menu().hide();
     }
@@ -655,6 +683,7 @@ void BScreen::hideWindowMenus(const FluxboxWindow* except) {
 
 
 void BScreen::reconfigure() {
+
     m_windowtheme->setFocusedAlpha(*resource.focused_alpha);
     m_windowtheme->setUnfocusedAlpha(*resource.unfocused_alpha);
     m_menutheme->setAlpha(*resource.menu_alpha);
@@ -679,17 +708,26 @@ void BScreen::reconfigure() {
     m_menutheme->setDelayOpen(*resource.menu_delay);
     m_menutheme->setDelayClose(*resource.menu_delay_close);
 
-    // setup windowtheme, toolbartheme for antialias
-    winFrameTheme().font().setAntialias(*resource.antialias);
-    m_menutheme->titleFont().setAntialias(*resource.antialias);
-    m_menutheme->frameFont().setAntialias(*resource.antialias);
-
     renderGeomWindow();
     renderPosWindow();
 
+    // realize the number of workspaces from the init-file
+    const int nr_ws = *resource.workspaces;
+    if (nr_ws > m_workspaces_list.size()) {
+        while(nr_ws != m_workspaces_list.size()) {
+            addWorkspace();
+        }
+    } else if (nr_ws < m_workspaces_list.size()) {
+        while(nr_ws != m_workspaces_list.size()) {
+            removeLastWorkspace();
+        }
+    }
+
     //reconfigure menus
     m_workspacemenu->reconfigure();
     m_configmenu->reconfigure();
+    // recreate window menu
+    m_windowmenu.reset(MenuCreator::createMenuType("windowmenu", screenNumber()));
 
     // We need to check to see if the timestamps
     // changed before we actually can restore the menus 
@@ -770,7 +808,7 @@ void BScreen::addIcon(FluxboxWindow *w) {
         return;
 
     // make sure we have a unique list
-    if (find(getIconList().begin(), getIconList().end(), w) != getIconList().end())
+    if (find(iconList().begin(), iconList().end(), w) != iconList().end())
         return;
 
     m_icon_list.push_back(w);
@@ -784,13 +822,13 @@ void BScreen::removeIcon(FluxboxWindow *w) {
     if (w == 0)
         return;
 
-    Icons::iterator erase_it = remove_if(getIconList().begin(),
-                                         getIconList().end(),
+    Icons::iterator erase_it = remove_if(iconList().begin(),
+                                         iconList().end(),
                                          bind2nd(equal_to<FluxboxWindow *>(), w));
     // no need to send iconlist signal if we didn't 
     // change the iconlist
     if (erase_it != m_icon_list.end()) {
-        getIconList().erase(erase_it);
+        iconList().erase(erase_it);
         m_iconlist_sig.notify();
     }
 }
@@ -826,35 +864,34 @@ void BScreen::removeClient(WinClient &client) {
     for_each(getWorkspacesList().begin(), getWorkspacesList().end(),
              mem_fun(&Workspace::updateClientmenu));
 
+    using namespace FbTk;
+
     // remove any grouping this is expecting
-    Groupables::iterator it = m_expecting_groups.begin();
-    Groupables::iterator it_end = m_expecting_groups.end();
-    for (; it != it_end; ++it) {
-        if (it->second == &client) {
-            m_expecting_groups.erase(it);
-            // it should only be in there a maximum of once
-            break;
-        }
-    }
+    Groupables::iterator erase_it = find_if(m_expecting_groups.begin(),
+                                            m_expecting_groups.end(),
+                                            Compose(bind2nd(equal_to<WinClient *>(), &client),
+                                                    Select2nd<Groupables::value_type>()));
+
+    if (erase_it != m_expecting_groups.end())
+        m_expecting_groups.erase(erase_it);
+
     // the client could be on icon menu so we update it
     //!! TODO: check this with the new icon menu
     //    updateIconMenu();
 
 }
 
-void BScreen::setAntialias(bool value) {
-    if (*resource.antialias == value)
-        return;
-    resource.antialias = value;
-    reconfigure();
-}
-
 int BScreen::addWorkspace() {
+
+    bool save_name = getNameOfWorkspace(m_workspaces_list.size()) != "" ? false : true;
     Workspace *wkspc = new Workspace(*this, m_layermanager, 
-                                     "",
+                                     getNameOfWorkspace(m_workspaces_list.size()),
                                      m_workspaces_list.size());
     m_workspaces_list.push_back(wkspc);
-    addWorkspaceName(wkspc->name().c_str()); // update names
+    
+    if (save_name)
+        addWorkspaceName(wkspc->name().c_str()); //update names
+
     saveWorkspaces(m_workspaces_list.size());
     
     updateNetizenWorkspaceCount();	
@@ -877,11 +914,13 @@ int BScreen::removeLastWorkspace() {
 
     //remove last workspace
     m_workspaces_list.pop_back();		
-    delete wkspc;
-
 
     updateNetizenWorkspaceCount();
     saveWorkspaces(m_workspaces_list.size());
+    // must be deleted after we send notify!!
+    // so we dont get bad pointers somewhere
+    // while processing the notify signal
+    delete wkspc;
 
     return m_workspaces_list.size();
 }
@@ -968,9 +1007,11 @@ void BScreen::sendToWorkspace(unsigned int id, FluxboxWindow *win, bool changeWS
             win->deiconify();
 
         // if the window isn't on current workspace, hide it
-        if (id != currentWorkspace()->workspaceID())
+        if (id != currentWorkspace()->workspaceID()) 
             win->withdraw(true);
 
+        windowMenu().hide();
+
         reassociateWindow(win, id, true);
 
         // if the window is on current workspace, show it.
@@ -1052,6 +1093,11 @@ void BScreen::updateNetizenWindowFocus() {
 
 
 void BScreen::updateNetizenWindowAdd(Window w, unsigned long p) {
+
+    // update the list of clients
+    m_clientlist_sig.notify();
+    
+    // and then send the signal to listeners
     Netizens::iterator it = m_netizen_list.begin();
     Netizens::iterator it_end = m_netizen_list.end();
     for (; it != it_end; ++it) {
@@ -1166,10 +1212,8 @@ FluxboxWindow *BScreen::createWindow(Window client) {
     FbTk::App::instance()->sync(false);
 
 
-    if (isKdeDockapp(client)) {
-        if (addKdeDockapp(client)) {            
-            return 0; // dont create a FluxboxWindow for this one
-        }
+    if (isKdeDockapp(client) && addKdeDockapp(client)) {
+        return 0; // dont create a FluxboxWindow for this one
     }
 
     WinClient *winclient = new WinClient(client, *this);
@@ -1183,8 +1227,6 @@ FluxboxWindow *BScreen::createWindow(Window client) {
         return 0;
     }
 
-    bool new_win = false;
-
     // check if it should be grouped with something else
     FluxboxWindow *win;
     if ((win = findGroupLeft(*winclient)) != 0) {
@@ -1200,8 +1242,6 @@ FluxboxWindow *BScreen::createWindow(Window client) {
                                     winFrameTheme(),
                                     *layerManager().getLayer(Fluxbox::instance()->getNormalLayer()));
             
-            new_win = true;
-
             if (!win->isManaged()) {
                 delete win;
                 return 0;
@@ -1215,11 +1255,7 @@ FluxboxWindow *BScreen::createWindow(Window client) {
         focused_list.push_front(&win->winClient());
     else
         focused_list.push_back(&win->winClient());
-    
-    if (new_win) {
-        Fluxbox::instance()->attachSignals(*win);
-    }
-
+ 
     // we also need to check if another window expects this window to the left
     // and if so, then join it.
     FluxboxWindow *otherwin = 0;
@@ -1236,37 +1272,26 @@ FluxboxWindow *BScreen::createWindow(Window client) {
 
 
 FluxboxWindow *BScreen::createWindow(WinClient &client) {
+
+    if (isKdeDockapp(client.window()) && addKdeDockapp(client.window())) {
+        return 0;
+    }
+
     FluxboxWindow *win = new FluxboxWindow(client,
                                            winFrameTheme(),
                                            *layerManager().getLayer(Fluxbox::instance()->getNormalLayer()));
 
-    if (isKdeDockapp(client.window())) {
-        if (addKdeDockapp(client.window())) {
-            // we need to save old handler and readd it later
-            FbTk::EventManager *evm = FbTk::EventManager::instance();
-            FbTk::EventHandler *evh = evm->find(client.window());
-            delete win;
-            evm->add(*evh, client.window());
-            return 0;
-        }
-    } else {
-
 #ifdef SLIT
-        if (win->initialState() == WithdrawnState && slit() != 0) {
-            slit()->addClient(win->clientWindow());
-        }
-#endif // SLIT
+    if (win->initialState() == WithdrawnState && slit() != 0) {
+        slit()->addClient(win->clientWindow());
     }
+#endif // SLIT
                  
 
     if (!win->isManaged()) {
         delete win;
         return 0;
     }
-    // don't add to focused_list, as it should already be in there (since the
-    // WinClient already exists).
-    
-    Fluxbox::instance()->attachSignals(*win);
 
     m_clientlist_sig.notify();
 
@@ -1333,7 +1358,7 @@ void BScreen::reassociateWindow(FluxboxWindow *w, unsigned int wkspc_id,
     if (w == 0)
         return;
 
-    if (wkspc_id >= getCount())
+    if (wkspc_id >= numberOfWorkspaces())
         wkspc_id = currentWorkspace()->workspaceID();
 
     if (!w->isIconic() && w->workspaceNumber() == wkspc_id)
@@ -1350,7 +1375,10 @@ void BScreen::reassociateWindow(FluxboxWindow *w, unsigned int wkspc_id,
         // gets updated
         m_clientlist_sig.notify(); 
     } else if (ignore_sticky || ! w->isStuck()) {
-        getWorkspace(w->workspaceNumber())->removeWindow(w, true);
+        // fresh windows have workspaceNumber == -1, which leads to
+        // an invalid workspace (unsigned int)
+        if (getWorkspace(w->workspaceNumber()))
+            getWorkspace(w->workspaceNumber())->removeWindow(w, true);
         getWorkspace(wkspc_id)->addWindow(*w);
         // see comment above
         m_clientlist_sig.notify();
@@ -1388,7 +1416,7 @@ void BScreen::nextFocus(int opts) {
                 break;
             }
 
-            FluxboxWindow *fbwin = (*it)->m_win;
+            FluxboxWindow *fbwin = (*it)->fbwindow();
             if (fbwin && !fbwin->isIconic() &&
                 (fbwin->isStuck() 
                  || fbwin->workspaceNumber() == currentWorkspaceID())) {
@@ -1483,7 +1511,7 @@ void BScreen::prevFocus(int opts) {
                 break;
             }
 
-            FluxboxWindow *fbwin = (*it)->m_win;
+            FluxboxWindow *fbwin = (*it)->fbwindow();
             if (fbwin && !fbwin->isIconic() &&
                 (fbwin->isStuck() 
                  || fbwin->workspaceNumber() == currentWorkspaceID())) {
@@ -1667,9 +1695,11 @@ void BScreen::dirFocus(FluxboxWindow &win, const FocusDir dir) {
 }
 void BScreen::initMenus() {
     m_workspacemenu.reset(MenuCreator::createMenuType("workspacemenu", screenNumber()));
+    m_windowmenu.reset(MenuCreator::createMenuType("windowmenu", screenNumber()));
     initMenu();
 }
 
+
 void BScreen::initMenu() {
 	
     if (m_rootmenu.get()) {
@@ -1704,7 +1734,7 @@ void BScreen::initMenu() {
                            exit_fb);
     }
 
-    m_rootmenu->update();
+    m_rootmenu->updateMenu();
 }
 
 
@@ -1714,14 +1744,12 @@ void BScreen::addConfigMenu(const char *label, FbTk::Menu &menu) {
 }
 
 void BScreen::removeConfigMenu(FbTk::Menu &menu) {
-    Configmenus::iterator it = m_configmenu_list.begin();
-    Configmenus::iterator it_end = m_configmenu_list.end();
-    for (; it != it_end; ++it) {
-        if (it->second == &menu) {
-            m_configmenu_list.erase(it);
-            break;
-        }
-    }
+    Configmenus::iterator erase_it = find_if(m_configmenu_list.begin(),
+                                             m_configmenu_list.end(),
+                                             FbTk::Compose(bind2nd(equal_to<FbTk::Menu *>(), &menu),
+                                                           FbTk::Select2nd<Configmenus::value_type>()));
+    if (erase_it != m_configmenu_list.end())
+        m_configmenu_list.erase(erase_it);
     setupConfigmenu(*m_configmenu.get());
 
 }    
@@ -1747,15 +1775,16 @@ void BScreen::setupConfigmenu(FbTk::Menu &menu) {
 
 #define _FOCUSITEM(a, b, c, d, e) focus_menu->insert(new FocusModelMenuItem(_FBTEXT(a, b, c, d), *this, e, save_and_reconfigure))
 
-    _FOCUSITEM(Configmenu, ClickToFocus,
+    _FOCUSITEM(Configmenu, ClickFocus,
                "Click To Focus", "Click to focus",
-               CLICKTOFOCUS);
-    _FOCUSITEM(Configmenu, SloppyFocus,
-               "Sloppy Focus", "Sloppy Focus",
-               SLOPPYFOCUS);
-    _FOCUSITEM(Configmenu, SemiSloppyFocus,
-               "Semi Sloppy Focus", "Semi Sloppy Focus",
-               SEMISLOPPYFOCUS);
+               CLICKFOCUS);
+    _FOCUSITEM(Configmenu, MouseFocus,
+               "Mouse Focus", "Mouse Focus",
+               MOUSEFOCUS);
+
+    focus_menu->insert(new TabFocusModelMenuItem("ClickTabFocus", *this, CLICKTABFOCUS, save_and_reconfigure));
+    focus_menu->insert(new TabFocusModelMenuItem("MouseTabFocus", *this, MOUSETABFOCUS, save_and_reconfigure));
+    
 #undef _FOCUSITEM
 
     focus_menu->insert(new BoolMenuItem(_FBTEXT(Configmenu, 
@@ -1765,7 +1794,7 @@ void BScreen::setupConfigmenu(FbTk::Menu &menu) {
                                         *resource.auto_raise,
                                         save_and_reconfigure));
 
-    focus_menu->update();
+    focus_menu->updateMenu();
 
     menu.insert(focusmenu_label, focus_menu);
 #ifdef SLIT
@@ -1809,12 +1838,6 @@ void BScreen::setupConfigmenu(FbTk::Menu &menu) {
     _BOOLITEM(Configmenu, ClickRaises,
               "Click Raises", "Click Raises", 
               *resource.click_raises, saverc_cmd);
-#ifdef USE_XFT
-    // setup antialias cmd to reload style and save resource on toggle
-    _BOOLITEM(Configmenu, AntiAlias,
-              "AntiAlias", "Use Anti-aliased fonts",
-              *resource.antialias, save_and_reconfigure);
-#endif // USE_XFT
 
 #ifdef HAVE_XRENDER
     if (FbTk::Transparent::haveRender() ||
@@ -1831,28 +1854,28 @@ void BScreen::setupConfigmenu(FbTk::Menu &menu) {
         }
 
         FbTk::MenuItem *focused_alpha_item = new IntResMenuItem(_FBTEXT(Configmenu, FocusedAlpha, "Focused Window Alpha", "Transparency level of the focused window"),
-                    resource.focused_alpha, 0, 255);
+                    resource.focused_alpha, 0, 255, *alpha_menu);
         focused_alpha_item->setCommand(saverc_cmd);
         alpha_menu->insert(focused_alpha_item);
 
         FbTk::MenuItem *unfocused_alpha_item = new IntResMenuItem(_FBTEXT(Configmenu, UnfocusedAlpha, "Unfocused Window Alpha", "Transparency level of unfocused windows"),
-                    resource.unfocused_alpha, 0, 255);
+                    resource.unfocused_alpha, 0, 255, *alpha_menu);
         unfocused_alpha_item->setCommand(saverc_cmd);
         alpha_menu->insert(unfocused_alpha_item);
 
         FbTk::MenuItem *menu_alpha_item = new IntResMenuItem(_FBTEXT(Configmenu, MenuAlpha, "Menu Alpha", "Transparency level of menu"),
-                    resource.menu_alpha, 0, 255);
+                    resource.menu_alpha, 0, 255, *alpha_menu);
         menu_alpha_item->setCommand(saverc_cmd);
         alpha_menu->insert(menu_alpha_item);
 
-        alpha_menu->update();
+        alpha_menu->updateMenu();
         menu.insert(alphamenu_label, alpha_menu);
     }
 #endif // HAVE_XRENDER
 #undef _BOOLITEM
 
     // finaly update menu 
-    menu.update();
+    menu.updateMenu();
 }
 
 
@@ -1974,21 +1997,21 @@ void BScreen::setLayer(FbTk::XLayerItem &item, int layernum) {
  Goes to the workspace "right" of the current
 */
 void BScreen::nextWorkspace(const int delta) {
-    changeWorkspaceID( (currentWorkspaceID() + delta) % getCount());
+    changeWorkspaceID( (currentWorkspaceID() + delta) % numberOfWorkspaces());
 }
 
 /**
  Goes to the workspace "left" of the current
 */
 void BScreen::prevWorkspace(const int delta) {
-    changeWorkspaceID( (currentWorkspaceID() - delta + getCount()) % getCount());
+    changeWorkspaceID( (currentWorkspaceID() - delta + numberOfWorkspaces()) % numberOfWorkspaces());
 }
 
 /**
  Goes to the workspace "right" of the current
 */
 void BScreen::rightWorkspace(const int delta) {
-    if (currentWorkspaceID()+delta < getCount())
+    if (currentWorkspaceID()+delta < numberOfWorkspaces())
         changeWorkspaceID(currentWorkspaceID()+delta);
 }
 
@@ -2134,7 +2157,7 @@ void BScreen::notifyReleasedKeys(XKeyEvent &ke) {
  */
 WinClient *BScreen::getLastFocusedWindow(int workspace) {
     if (focused_list.empty()) return 0;
-    if (workspace < 0 || workspace >= (int) getCount())
+    if (workspace < 0 || workspace >= (int) numberOfWorkspaces())
         return focused_list.front();
 
     FocusedWindows::iterator it = focused_list.begin();    
@@ -2221,7 +2244,7 @@ FluxboxWindow *BScreen::findGroupRight(WinClient &winclient) {
         other->getGroupLeftWindow() != None)
         return 0;
 
-    return other->m_win;
+    return other->fbwindow();
 }
 void BScreen::initXinerama() {
 #ifdef XINERAMA
@@ -2372,16 +2395,12 @@ pair<int,int> BScreen::clampToHead(int head, int x, int y, int w, int h) const {
 // this can be gone and a consistent interface for the two used
 // on the actual objects
 
-
-#ifdef SLIT
-template <>
-int BScreen::getOnHead<Slit>(Slit &slit) {
-    return 0;
+template<>
+void BScreen::setOnHead<FluxboxWindow>(FluxboxWindow& win, int head) {
+    if (head > 0 && head <= numHeads()) {
+        int current_head = getHead(win.fbWindow());
+        win.move(getHeadX(head) + win.frame().x() - getHeadX(current_head),
+                 getHeadY(head) + win.frame().y() - getHeadY(current_head));
+    }
 }
 
-template <>
-void BScreen::setOnHead<Slit>(Slit &slit, int head) {
-    //    slit.saveOnHead(head);
-    slit.reconfigure();
-}
-#endif // SLIT
