@@ -1,5 +1,5 @@
 // Slit.cc for fluxbox
-// Copyright (c) 2002 - 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Slit.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Slit.cc 3747 2004-12-02 15:30:59Z mathias $
+// $Id: Slit.cc 4062 2005-06-23 03:07:25Z fluxgen $
 
 #include "Slit.hh"
 
@@ -295,6 +295,8 @@ Slit::Slit(BScreen &scr, FbTk::XLayer &layer, const char *filename)
     scr.reconfigureSig().attach(this); // if alpha changed (we disablethis signal when we get theme change sig)
 
     frame.pixmap = None;
+    // move the frame out of sight for a moment
+    frame.window.move(-frame.window.width(), -frame.window.height());
     // setup timer
     m_timer.setTimeout(200); // default timeout
     m_timer.fireOnce(true);
@@ -311,16 +313,18 @@ Slit::Slit(BScreen &scr, FbTk::XLayer &layer, const char *filename)
     }
 
     m_layeritem.reset(new FbTk::XLayerItem(frame.window, layer));
-    m_layermenu.reset(new LayerMenu<Slit>(scr.menuTheme(),
-                                          scr.imageControl(),
-                                          *scr.layerManager().
-                                          getLayer(Fluxbox::instance()->getMenuLayer()), 
-                                          this,
-                                          true));
+
+    m_layermenu.reset(new LayerMenu(scr.menuTheme(),
+                                    scr.imageControl(),
+                                    *scr.layerManager().
+                                    getLayer(Fluxbox::instance()->getMenuLayer()), 
+                                    this,
+                                    true));
+    m_layermenu->setLabel(_FBTEXT(Slit, Layer, "Slit Layer", "Title of Slit Layer Menu"));
+
     moveToLayer((*m_rc_layernum).getNum());
 
-    if (m_layermenu.get())
-        m_layermenu->setLabel(_FBTEXT(Slit, Layer, "Slit Layer", "Title of Slit Layer Menu"));
+
 
     // Get client list for sorting purposes
     loadClientList(filename);
@@ -357,52 +361,53 @@ void Slit::updateStrut() {
         return;
     }
 
-
+    unsigned int bw = m_slit_theme->borderWidth();
     int left = 0, right = 0, top = 0, bottom = 0;
     switch (placement()) {
     case TOPLEFT:
         if (direction() == HORIZONTAL)
-            top = height();
+            top = height() + 2 * bw;
         else
-            left = width();
+            left = width() + 2 * bw;
         break;
     case TOPCENTER:
         if (direction() == HORIZONTAL)
-            top = height();
+            top = height() + 2 * bw;
         break;
     case TOPRIGHT:
         if (direction() == HORIZONTAL)
-            top = height();
+            top = height() + 2 * bw;
         else
-            right = width();
+            right = width() + 2 * bw;
         break;
     case BOTTOMLEFT:
         if (direction() == HORIZONTAL)
-            bottom = height();
+            bottom = height() + 2 * bw;
         else
-            left = width();
+            left = width() + 2 * bw;
         break;
     case BOTTOMCENTER:
         // would be strange to have it request size on vertical direction
         // each time we add a client
         if (direction() == HORIZONTAL)
-            bottom = height();
+            bottom = height() + 2 * bw;
         break;
     case BOTTOMRIGHT:
         if (direction() == HORIZONTAL)
-            bottom = height();
+            bottom = height() + 2 * bw;
         else
-            right = width();
+            right = width() + 2 * bw;
         break;
     case LEFTCENTER:
         if (direction() == VERTICAL)
-            left = width();        
+            left = width() + 2 * bw;
         break;
     case RIGHTCENTER:
         if (direction() == VERTICAL)
-            right = width();
+            right = width() + 2 * bw;
         break;
     }
+
     m_strut = screen().requestStrut(getOnHead(), left, right, top, bottom);
     screen().updateAvailableWorkspaceArea();
 }
@@ -910,11 +915,17 @@ void Slit::clientUp(SlitClient* client) {
     if (!client || m_client_list.size() < 2)
         return;
     
+    if (client == m_client_list.front()) {
+        cycleClientsUp();
+        return;
+    }
+    
     SlitClients::iterator it = m_client_list.begin();
-
     for(it++; it != m_client_list.end(); it++) {
         if ((*it) == client) {
-            swap(*it, *(it--));
+            SlitClients::iterator prev = it;
+            prev--;
+            iter_swap(it, prev);
             reconfigure();
             break;
         }
@@ -924,11 +935,18 @@ void Slit::clientUp(SlitClient* client) {
 void Slit::clientDown(SlitClient* client) {
     if (!client || m_client_list.size() < 2)
         return;
-
+    
+    if (client == m_client_list.back()) {
+        cycleClientsDown();
+        return;
+    }
+    
     SlitClients::reverse_iterator it = m_client_list.rbegin();
     for(it++; it != m_client_list.rend(); it++) {
         if ((*it) == client) {
-            swap(*it, *(it--));
+            SlitClients::reverse_iterator next = it;
+            next--;
+            iter_swap(it, next);
             reconfigure();
             break;
         }
@@ -1069,7 +1087,6 @@ void Slit::exposeEvent(XExposeEvent &ev) {
     // we don't need to clear the entire window 
     // just the are that gets exposed
     frame.window.clearArea(ev.x, ev.y, ev.width, ev.height);
-    frame.window.updateTransparent(ev.x, ev.y, ev.width, ev.height);
 }
 
 void Slit::update(FbTk::Subject *subj) {
@@ -1078,7 +1095,6 @@ void Slit::update(FbTk::Subject *subj) {
 
 void Slit::clearWindow() {
     frame.window.clear();
-    frame.window.updateTransparent();
 }
 
 void Slit::toggleHidden() {
@@ -1161,7 +1177,7 @@ void Slit::updateClientmenu() {
                                      "Save SlitList", "Saves the current order in the slit"), 
                              savecmd);
 
-    m_clientlist_menu.update();
+    m_clientlist_menu.updateMenu();
 }
 
 void Slit::saveClientList() {
@@ -1243,7 +1259,7 @@ void Slit::setupMenu() {
     FbTk::MenuItem *alpha_menuitem = 
         new IntResMenuItem(_FBTEXT(Common, Alpha, "Alpha", "Transparency level"),
                            m_rc_alpha,
-                           0, 255);
+                           0, 255, m_slitmenu);
     // setup command for alpha value
     MacroCommand *alpha_macrocmd = new MacroCommand(); 
     RefCount<Command> alpha_cmd(new SimpleCommand<Slit>(*this, &Slit::updateAlpha));
@@ -1258,7 +1274,7 @@ void Slit::setupMenu() {
                                           *this,
                                           save_and_reconfigure));
     m_slitmenu.insert(_FBTEXT(Slit, ClientsMenu, "Clients", "Slit client menu"), &m_clientlist_menu);
-    m_slitmenu.update();
+    m_slitmenu.updateMenu();
 
     // setup sub menu
     m_placement_menu.setLabel(_FBTEXT(Slit, Placement, "Slit Placement", "Slit Placement"));
@@ -1300,7 +1316,7 @@ void Slit::setupMenu() {
     }
 
     // finaly update sub menu
-    m_placement_menu.update();
+    m_placement_menu.updateMenu();
 }
 
 void Slit::moveToLayer(int layernum) {
