@@ -1,5 +1,5 @@
 // SystemTray.cc
-// Copyright (c) 2003-2005 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: SystemTray.cc 3838 2005-01-14 09:58:16Z mathias $
+// $Id: SystemTray.cc 4023 2005-05-17 11:24:50Z simonb $
 
 #include "SystemTray.hh"
 
@@ -326,7 +326,6 @@ void SystemTray::removeClient(Window win) {
 
 void SystemTray::exposeEvent(XExposeEvent &event) {
     m_window.clear();
-    update(0);
 }
 
 void SystemTray::handleEvent(XEvent &event) {
@@ -348,8 +347,12 @@ void SystemTray::handleEvent(XEvent &event) {
                 static_cast<unsigned int>(event.xconfigure.height) != (*it)->height()) {
                 // the position might differ so we update from our local
                 // copy of position
-                (*it)->moveResize((*it)->x(), (*it)->y(),
+                XMoveResizeWindow(FbTk::App::instance()->display(), (*it)->window(), 
+                                  (*it)->x(), (*it)->y(),
                                   (*it)->width(), (*it)->height());
+
+                // this was why gaim wasn't centring the icon
+                (*it)->sendConfigureNotify(0, 0, (*it)->width(), (*it)->height());
             }
             // so toolbar know that we changed size
             resizeSig().notify();
@@ -359,19 +362,23 @@ void SystemTray::handleEvent(XEvent &event) {
 }
 
 void SystemTray::rearrangeClients() {
+    const unsigned int h = height();
+    const unsigned int bw = m_theme.border().width();
+    int final_size = m_clients.size()*h + bw;
+    resize(final_size, h);
+    update(0);
+
     // move and resize clients
     ClientList::iterator client_it = m_clients.begin();
     ClientList::iterator client_it_end = m_clients.end();
-    int next_x = 0;
-    const unsigned int h = height();
-    const unsigned int b = m_theme.border().width();
+    int next_x = bw;
     for (; client_it != client_it_end;
-         ++client_it, next_x += h - 2 * b) {
-        (*client_it)->moveResize(next_x, b, h - b, h - b);
+         ++client_it, next_x += h+bw) {
+        (*client_it)->moveResize(next_x, bw, h, h);
+        (*client_it)->sendConfigureNotify(next_x, bw, h, h);
     }
 
-    resize(next_x, height());
-    update(0);
+    client_it = m_clients.begin();
 }
 
 void SystemTray::removeAllClients() {
