@@ -1,5 +1,5 @@
 // Workspace.cc for Fluxbox
-// Copyright (c) 2001 - 2004 Henrik Kinnunen (fluxgen[at]users.sourceforge.net)
+// Copyright (c) 2001 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Workspace.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Workspace.cc 3709 2004-11-22 08:30:23Z mathias $
+// $Id: Workspace.cc 4062 2005-06-23 03:07:25Z fluxgen $
 
 #include "Workspace.hh"
 
@@ -31,6 +31,7 @@
 #include "Window.hh"
 #include "WinClient.hh"
 #include "FbWinFrame.hh"
+#include "WindowCmd.hh"
 
 #include "FbTk/I18n.hh"
 #include "FbTk/MenuItem.hh"
@@ -83,18 +84,32 @@ int countTransients(const WinClient &client) {
 class ClientMenuItem:public FbTk::MenuItem {
 public:
     ClientMenuItem(WinClient &client):
-        FbTk::MenuItem(client.title().c_str(), client.fbwindow() ? &client.fbwindow()->menu() : 0),
+        FbTk::MenuItem(client.title().c_str(), &client.screen().windowMenu()),
         m_client(client) {
         
     }
+    FbTk::Menu *submenu() { return &m_client.screen().windowMenu(); }
+    const FbTk::Menu *submenu() const { return &m_client.screen().windowMenu(); }
+
+    void showSubmenu() {
+        WindowCmd<void>::setWindow(m_client.fbwindow());
+        FbTk::MenuItem::showSubmenu();
+    }
+
     void click(int button, int time) {
         if (m_client.fbwindow() == 0)
             return;
         FluxboxWindow &win = *m_client.fbwindow();
 
-        win.screen().changeWorkspaceID(win.workspaceNumber()); 
-        win.setCurrentClient(m_client);
-        win.raiseAndFocus();
+        // fetch the window to the current workspace
+        if (button == 2 && win.screen().currentWorkspaceID() != win.workspaceNumber()) {
+            win.menu().hide();
+            win.screen().sendToWorkspace(win.screen().currentWorkspaceID(), &win, true);
+        } else { // warp to the workspace of the window
+            win.screen().changeWorkspaceID(win.workspaceNumber()); 
+            win.setCurrentClient(m_client);
+            win.raiseAndFocus();
+        }
     }
 
     const std::string &label() const { return m_client.title(); }
@@ -353,12 +368,12 @@ bool Workspace::loadGroups(const std::string &filename) {
 }
 
 void Workspace::update(FbTk::Subject *subj) {
-    menu().update();
+    menu().updateMenu();
 }
 
 
 void Workspace::setName(const std::string &name) {
-    if (!name.empty()) {
+    if (!name.empty() && name != "") {
         m_name = name;
     } else { //if name == 0 then set default name from nls
         _FB_USES_NLS;
@@ -369,11 +384,11 @@ void Workspace::setName(const std::string &name) {
                 m_id + 1); //m_id starts at 0
         m_name = tname;
     }
-	
+    
     screen().updateWorkspaceNamesAtom();
 	
     menu().setLabel(m_name.c_str());
-    menu().update();
+    menu().updateMenu();
 }
 
 /**
@@ -407,7 +422,7 @@ void Workspace::updateClientmenu() {
             menu().insert(new ClientMenuItem(*(*client_it)));
     }
 
-    menu().update();
+    menu().updateMenu();
 }
 
 void Workspace::placeWindow(FluxboxWindow &win) {
