@@ -1,6 +1,6 @@
 // main.cc for Fluxbox Window manager
-// Copyright (c) 2001 - 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
-//               and 2003 Simon Bowden (rathnor at users.sourceforge.net)
+// Copyright (c) 2001 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+//               and 2003-2005 Simon Bowden (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: main.cc 3793 2004-12-21 23:42:09Z mathias $
+// $Id: main.cc 4095 2005-08-31 20:15:01Z fluxgen $
 
 #include "fluxbox.hh"
 #include "version.h"
@@ -63,6 +63,9 @@ using namespace std;
 void showInfo(ostream &ostr) {
     _FB_USES_NLS;
     ostr<<_FBTEXT(Common, FluxboxVersion, "Fluxbox version", "Fluxbox version heading")<<": "<<__fluxbox_version<<endl;
+
+    if (strlen(svnversion()) > 0)
+        ostr << _FBTEXT(Common, SvnRevision, "SVN Revision", "Revision number in SVN repositary") << ": " << svnversion() << endl;
 #if defined(__DATE__) && defined(__TIME__)
     ostr<<_FBTEXT(Common, Compiled, "Compiled", "Time fluxbox was compiled")<<": "<<__DATE__<<" "<<__TIME__<<endl;
 #endif
@@ -81,6 +84,10 @@ void showInfo(ostream &ostr) {
     ostr<<_FBTEXT(Common, DefaultKeyFile, "    keys", "default key file (right aligned - make sure same width as other default values)")<<": "<<DEFAULTKEYSFILE<<endl;
     ostr<<_FBTEXT(Common, DefaultInitFile, "    init", "default init file (right aligned - make sure same width as other default values)")<<": "<<DEFAULT_INITFILE<<endl;
 
+#ifdef NLS
+    ostr<<_FBTEXT(Common, DefaultLocalePath, "    nls", "location for localization files (right aligned - make sure same width as other default values)")<<": "<<LOCALEPATH<<endl;
+#endif
+
     const char NOT[] = "-";
     ostr<<endl<<
         _FBTEXT(Common, CompiledOptions, "Compiled options", "Options used when compiled")
@@ -200,7 +207,7 @@ int main(int argc, char **argv) {
             }
         } else if (strcmp(argv[i], "-version") == 0 || strcmp(argv[i], "-v") == 0) {
             // print current version string
-            cout<<"Fluxbox "<<__fluxbox_version<<" : (c) 2001-2004 Henrik Kinnunen "<<endl<<endl;
+            cout<<"Fluxbox "<<__fluxbox_version<<" : (c) 2001-2005 Henrik Kinnunen "<<endl<<endl;
             exit(0);
         } else if (strcmp(argv[i], "-log") == 0 ) {
             if (i + 1 >= argc) {
@@ -214,6 +221,7 @@ int main(int argc, char **argv) {
                            "Fluxbox %s : (c) %s Henrik Kinnunen\n"
                            "Website: http://www.fluxbox.org/ \n\n"
                            "\t-display <string>\t\tuse display connection.\n"
+                           "\t-screen <all|int,int,int>\trun on specified screens only.\n"
                            "\t-rc <string>\t\t\tuse alternate resource file.\n"
                            "\t-version\t\t\tdisplay version and exit.\n"
                            "\t-info\t\t\t\tdisplay some useful information.\n"
@@ -221,7 +229,7 @@ int main(int argc, char **argv) {
                            "\t-help\t\t\t\tdisplay this help text and exit.\n\n",
 
                            "Main usage string. Please lay it out nicely. There is one %s that is given the version"),
-                   __fluxbox_version, "2001-2004");
+                   __fluxbox_version, "2001-2005");
             exit(0);
         } else if (strcmp(argv[i], "-info") == 0 || strcmp(argv[i], "-i") == 0) {
             showInfo(cout);
