@@ -1,5 +1,5 @@
 // FbRun.cc
-// Copyright (c) 2002-2003 Henrik Kinnunen (fluxgen<at>users.sourceforge.net)
+// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbRun.cc 3700 2004-11-19 11:37:27Z mathias $
+// $Id: FbRun.cc 4001 2005-05-09 07:20:17Z mathias $
 
 #include "FbRun.hh"
 
@@ -27,7 +27,7 @@
 #include "EventManager.hh"
 #include "Color.hh"
 #include "KeyUtil.hh"
-#include "Directory.hh"
+#include "FileUtil.hh"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -117,7 +117,7 @@ void FbRun::run(const std::string &command) {
     // fork and execute program
     if (!fork()) {
         setsid();
-        execl("/bin/sh", "/bin/sh", "-c", command.c_str(), 0);
+        execl("/bin/sh", "/bin/sh", "-c", command.c_str(), static_cast<void*>(NULL));
         exit(0); //exit child
     }
 
@@ -140,20 +140,27 @@ void FbRun::run(const std::string &command) {
             }
         }
 
-        // now m_current_history_item points at the duplicate, or
-        // at m_history.size() if no duplicate
         fstream inoutfile(m_history_file.c_str(), ios::in|ios::out);
         if (inoutfile) {
-            int i = 0;
-            // read past history items before current
-            for (string line; !inoutfile.eof() && i < m_current_history_item; i++)
-                getline(inoutfile, line);
-            // write the history items that come after current
-            for (i++; i < m_history.size(); i++)
-                inoutfile<<m_history[i]<<endl;
-            
-            // and append the current one back to the end
-            inoutfile<<text()<<endl;
+            // now m_current_history_item points at the duplicate, or
+            // at m_history.size() if no duplicate
+            if (m_current_history_item != m_history.size()) {
+                int i = 0;
+                // read past history items before current
+                for (; inoutfile.good() && i < m_current_history_item; i++)
+                    inoutfile.ignore(1, '\n');
+
+                // write the history items that come after current
+                for (i++; i < m_history.size(); i++)
+                    inoutfile<<m_history[i]<<endl;
+                
+            } else {
+                // set put-pointer at end of file
+                inoutfile.seekp(0, ios::end);
+            }
+            // append current command to the file
+            inoutfile<<command<<endl;
+
         } else
             cerr<<"FbRun Warning: Can't write command history to file: "<<m_history_file<<endl;
     }
@@ -216,7 +223,7 @@ void FbRun::redrawLabel() {
 }
 
 void FbRun::keyPressEvent(XKeyEvent &ke) {
-    // strip numlock, capslock and scrolllock mask
+    
     ke.state = FbTk::KeyUtil::instance().cleanMods(ke.state);
 
     int cp= cursorPosition();
@@ -225,10 +232,11 @@ void FbRun::keyPressEvent(XKeyEvent &ke) {
     char keychar[1];
     XLookupString(&ke, keychar, 1, &ks, 0);
     // a modifier key by itself doesn't do anything
-    if (IsModifierKey(ks)) return;
+    if (IsModifierKey(ks)) 
+        return;
 
-    if (ke.state) { // a modifier key is down
-        if (ke.state == ControlMask) {
+    if (FbTk::KeyUtil::instance().isolateModifierMask(ke.state)) { // a modifier key is down
+        if ((ke.state & ControlMask) == ControlMask) {
             switch (ks) {
             case XK_p:
                 prevHistoryItem();
@@ -241,7 +249,7 @@ void FbRun::keyPressEvent(XKeyEvent &ke) {
                 setCursorPosition(cp);
                 break;
             }
-        } else if (ke.state == (Mod1Mask | ShiftMask)) {
+        } else if ((ke.state & (Mod1Mask|ShiftMask)) == (Mod1Mask | ShiftMask)) {
             switch (ks) {
             case XK_less:
                 firstHistoryItem();
@@ -335,7 +343,7 @@ void FbRun::tabCompleteHistory() {
     } else {
         unsigned int nr= 0;
         int history_item = m_current_history_item - 1;
-        string prefix = text().substr(0, cursorPosition());
+        string prefix = text().substr(0, textStartPos() + cursorPosition());
         while (history_item != m_current_history_item && nr++ < m_history.size()) {
             if (history_item <= -1 )
                 history_item= m_history.size() - 1;
@@ -354,7 +362,7 @@ void FbRun::tabCompleteApps() {
   
     static bool first_run= true;
     static string saved_prefix= "";
-    string prefix= text().substr(0, cursorPosition());
+    string prefix= text().substr(0, textStartPos() + cursorPosition());
     FbTk::Directory dir;
 
     bool add_dirs= false;
@@ -393,18 +401,18 @@ void FbRun::tabCompleteApps() {
                                     filename;
 
                         // directories in dirmode ?
-                        if (add_dirs && dir.isDirectory(fncomplete) &&
+                        if (add_dirs && FbTk::FileUtil::isDirectory(fncomplete.c_str()) &&
                             filename != ".." && filename != ".") {
                             m_apps.push_back(fncomplete); 
                         // executables in dirmode ?
-                        } else if (add_dirs && dir.isRegularFile(fncomplete) && 
-                                   dir.isExecutable(fncomplete) && 
+                        } else if (add_dirs && FbTk::FileUtil::isRegularFile(fncomplete.c_str()) && 
+                                   FbTk::FileUtil::isExecutable(fncomplete.c_str()) && 
                                    (prefix == "" || 
                                     fncomplete.substr(0, prefix.size()) == prefix)) {
                             m_apps.push_back(fncomplete);
                         // executables in $PATH ?
-                        } else if (dir.isRegularFile(fncomplete) && 
-                                   dir.isExecutable(fncomplete) && 
+                        } else if (FbTk::FileUtil::isRegularFile(fncomplete.c_str()) && 
+                                   FbTk::FileUtil::isExecutable(fncomplete.c_str()) && 
                                    (prefix == "" || 
                                     filename.substr(0, prefix.size()) == prefix)) {
                             m_apps.push_back(filename);
@@ -440,7 +448,7 @@ void FbRun::tabCompleteApps() {
             }
             if (m_apps[apps_item].find(prefix) == 0) {
                 m_current_apps_item = apps_item;
-                if (add_dirs && FbTk::Directory::isDirectory(m_apps[m_current_apps_item]))
+                if (add_dirs && FbTk::FileUtil::isDirectory(m_apps[m_current_apps_item].c_str()))
                     setText(m_apps[m_current_apps_item] +  "/");
                 else
                     setText(m_apps[m_current_apps_item]);
