@@ -1,5 +1,5 @@
 // main.cc for FbRun
-// Copyright (c) 2002-2003 Henrik Kinnunen (fluxgen(at)linuxmail.org)
+// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: main.cc 3717 2004-11-23 11:37:39Z mathias $
+// $Id: main.cc 4043 2005-06-03 07:25:48Z mathias $
 
 #include "FbRun.hh"
 #include "App.hh"
@@ -119,7 +119,7 @@ int main(int argc, char **argv) {
         FbTk::App application(display_name.c_str());
         FbRun fbrun;
 
-        fbrun.setAntialias(antialias);
+        //fbrun.setAntialias(antialias);
 
         if (fontname.size() != 0) {
             if (!fbrun.loadFont(fontname.c_str())) {
@@ -140,8 +140,8 @@ int main(int argc, char **argv) {
             fbrun.resize(fbrun.width(), height);
         if (set_width)
             fbrun.resize(width, fbrun.height());
-        if (antialias)
-            fbrun.setAntialias(antialias);
+        //if (antialias)
+        //    fbrun.setAntialias(antialias);
         // expand and load command history
         string expanded_filename = FbTk::StringUtil::expandFilename(history_file);
         if (!fbrun.loadHistory(expanded_filename.c_str()))
@@ -149,7 +149,6 @@ int main(int argc, char **argv) {
 
         fbrun.setTitle(title);
         fbrun.setText(text);
-        fbrun.show();
 	
         if (near_mouse) {
             
@@ -208,6 +207,8 @@ int main(int argc, char **argv) {
         if (set_pos)
             fbrun.move(x, y);
 		
+        fbrun.show();
+
         application.eventLoop();
 
     } catch (string errstr) {
